# -*- coding: utf-8 -*-
import numpy as np

from ase.units import Hartree, Bohr

from gpaw.io import Reader
from gpaw.io import Writer
from gpaw.external import ConstantElectricField
from gpaw.lcaotddft.hamiltonian import KickHamiltonian
from gpaw.lcaotddft.utilities import read_uMM
from gpaw.lcaotddft.utilities import write_uMM
from gpaw.utilities.tools import tri2full


class KohnShamDecomposition(object):
    version = 1
    ulmtag = 'KSD'
    readwrite_attrs = ['fermilevel', 'only_ia', 'w_p', 'f_p', 'ia_p',
                       'P_p', 'dm_vp']

    def __init__(self, paw=None, filename=None):
        self.filename = filename
        self.has_initialized = False
        self.world = paw.world
        self.log = paw.log
        self.wfs = paw.wfs
        self.density = paw.density

        if self.wfs.bd.comm.size > 1:
            raise RuntimeError('Band parallelization is not supported')
        if len(self.wfs.kpt_u) > 1:
            raise RuntimeError('K-points are not supported')

        if filename is not None:
            self.read(filename)
            return

    def initialize(self, paw, min_occdiff=1e-3, only_ia=True):
        if self.has_initialized:
            return
        paw.initialize_positions()
        # paw.set_positions()

        if self.wfs.gd.pbc_c.any():
            self.C0_dtype = complex
        else:
            self.C0_dtype = float

        # Take quantities
        self.fermilevel = paw.occupations.get_fermi_level()
        self.S_uMM = []
        self.C0_unM = []
        self.eig_un = []
        self.occ_un = []
        for kpt in self.wfs.kpt_u:
            S_MM = kpt.S_MM
            assert np.max(np.absolute(S_MM.imag)) == 0.0
            S_MM = S_MM.real
            self.S_uMM.append(S_MM)

            C_nM = kpt.C_nM
            if self.C0_dtype == float:
                assert np.max(np.absolute(C_nM.imag)) == 0.0
                C_nM = C_nM.real
            self.C0_unM.append(C_nM)

            self.eig_un.append(kpt.eps_n)
            self.occ_un.append(kpt.f_n)

        # TODO: do the rest of the function with K-points

        # Construct p = (i, a) pairs
        u = 0
        Nn = self.wfs.bd.nbands
        eig_n = self.eig_un[u]
        occ_n = self.occ_un[u]

        self.only_ia = only_ia
        f_p = []
        w_p = []
        i_p = []
        a_p = []
        ia_p = []
        i0 = 0
        for i in range(i0, Nn):
            if only_ia:
                a0 = i + 1
            else:
                a0 = 0
            for a in range(a0, Nn):
                f = occ_n[i] - occ_n[a]
                if only_ia and f < min_occdiff:
                    continue
                w = eig_n[a] - eig_n[i]
                f_p.append(f)
                w_p.append(w)
                i_p.append(i)
                a_p.append(a)
                ia_p.append((i, a))
        f_p = np.array(f_p)
        w_p = np.array(w_p)
        i_p = np.array(i_p, dtype=int)
        a_p = np.array(a_p, dtype=int)
        ia_p = np.array(ia_p, dtype=int)

        # Sort according to energy difference
        p_s = np.argsort(w_p)
        f_p = f_p[p_s]
        w_p = w_p[p_s]
        i_p = i_p[p_s]
        a_p = a_p[p_s]
        ia_p = ia_p[p_s]

        Np = len(f_p)
        P_p = []
        for p in range(Np):
            P = np.ravel_multi_index(ia_p[p], (Nn, Nn))
            P_p.append(P)
        P_p = np.array(P_p)

        dm_vMM = []
        for v in range(3):
            direction = np.zeros(3, dtype=float)
            direction[v] = 1.0
            magnitude = 1.0
            cef = ConstantElectricField(magnitude * Hartree / Bohr, direction)
            kick_hamiltonian = KickHamiltonian(paw, cef)
            dm_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(
                kick_hamiltonian, paw.wfs, self.wfs.kpt_u[u],
                add_kinetic=False, root=-1)
            tri2full(dm_MM)  # TODO: do not use this
            dm_vMM.append(dm_MM)

        C0_nM = self.C0_unM[u]
        dm_vnn = []
        for v in range(3):
            dm_vnn.append(np.dot(C0_nM.conj(), np.dot(dm_vMM[v], C0_nM.T)))
        dm_vnn = np.array(dm_vnn)
        dm_vP = dm_vnn.reshape(3, -1)

        dm_vp = dm_vP[:, P_p]

        self.w_p = w_p
        self.f_p = f_p
        self.ia_p = ia_p
        self.P_p = P_p
        self.dm_vp = dm_vp

        self.has_initialized = True

    def write(self, filename):
        self.log('%s: Writing to %s' % (self.__class__.__name__, filename))
        writer = Writer(filename, self.world, mode='w',
                        tag=self.__class__.ulmtag)
        writer.write(version=self.__class__.version)

        wfs = self.wfs
        writer.write(ha=Hartree)
        write_uMM(wfs, writer, 'S_uMM', self.S_uMM)
        wfs.write_wave_functions(writer)
        wfs.write_eigenvalues(writer)
        wfs.write_occupations(writer)
        # write_unM(wfs, writer, 'C0_unM', self.C0_unM)
        # write_un(wfs, writer, 'eig_un', self.eig_un)
        # write_un(wfs, writer, 'occ_un', self.occ_un)

        for arg in self.readwrite_attrs:
            writer.write(arg, getattr(self, arg))

        writer.close()

    def read(self, filename):
        reader = Reader(filename)
        tag = reader.get_tag()
        if tag != self.__class__.ulmtag:
            raise RuntimeError('Unknown tag %s' % tag)
        version = reader.version
        if version != self.__class__.version:
            raise RuntimeError('Unknown version %s' % version)

        wfs = self.wfs
        self.S_uMM = read_uMM(wfs, reader, 'S_uMM')
        wfs.read_wave_functions(reader)
        wfs.read_eigenvalues(reader)
        wfs.read_occupations(reader)

        self.C0_unM = []
        self.eig_un = []
        self.occ_un = []
        for kpt in self.wfs.kpt_u:
            C_nM = kpt.C_nM
            self.C0_unM.append(C_nM)
            self.eig_un.append(kpt.eps_n)
            self.occ_un.append(kpt.f_n)

        for arg in self.readwrite_attrs:
            setattr(self, arg, getattr(reader, arg))

        reader.close()
        self.has_initialized = True

    def transform(self, rho_uMM):
        assert len(rho_uMM) == 1, 'K-points not implemented'
        u = 0
        C0_nM = self.C0_unM[u]
        S_MM = self.S_uMM[u]
        rho_MM = rho_uMM[u]
        # KS decomposition
        C0S_nM = np.dot(C0_nM, S_MM)
        rho_nn = np.dot(np.dot(C0S_nM, rho_MM), C0S_nM.T.conjugate())
        rho_P = rho_nn.reshape(-1)

        # Remove de-excitation terms
        rho_p = rho_P[self.P_p]
        if self.only_ia:
            rho_p *= 2

        rho_up = [rho_p]
        return rho_up

    def ialims(self):
        i_p = self.ia_p[:, 0]
        a_p = self.ia_p[:, 1]
        imin = np.min(i_p)
        imax = np.max(i_p)
        amin = np.min(a_p)
        amax = np.max(a_p)
        return imin, imax, amin, amax

    def M_p_to_M_ia(self, M_p):
        return self.M_ia_from_M_p(M_p)

    def M_ia_from_M_p(self, M_p):
        imin, imax, amin, amax = self.ialims()
        M_ia = np.zeros((imax - imin + 1, amax - amin + 1), dtype=M_p.dtype)
        for M, (i, a) in zip(M_p, self.ia_p):
            M_ia[i - imin, a - amin] = M
        return M_ia

    def plot_matrix(self, M_p):
        import matplotlib.pyplot as plt
        M_ia = self.M_ia_from_M_p(M_p)
        plt.imshow(M_ia, interpolation='none')
        plt.xlabel('a')
        plt.ylabel('i')

    def get_dipole_moment_contributions(self, rho_up):
        assert len(rho_up) == 1, 'K-points not implemented'
        u = 0
        rho_p = rho_up[u]
        dmrho_vp = - self.dm_vp * rho_p
        return dmrho_vp

    def get_dipole_moment(self, rho_up):
        assert len(rho_up) == 1, 'K-points not implemented'
        u = 0
        rho_p = rho_up[u]
        dm_v = - np.dot(self.dm_vp, rho_p)
        return dm_v

    def get_density(self, rho_up, density='comp'):
        from gpaw.lcaotddft.densitymatrix import get_density

        density_type = density
        assert len(rho_up) == 1, 'K-points not implemented'
        u = 0
        rho_p = rho_up[u]
        C0_nM = self.C0_unM[u]

        rho_ia = self.M_ia_from_M_p(rho_p)
        imin, imax, amin, amax = self.ialims()
        C0_iM = C0_nM[imin:(imax + 1)]
        C0_aM = C0_nM[amin:(amax + 1)]

        rho_MM = np.dot(C0_iM.T, np.dot(rho_ia, C0_aM.conj()))
        rho_MM = 0.5 * (rho_MM + rho_MM.T)

        return get_density(rho_MM, self.wfs, self.density, density_type, u)

    def get_contributions_table(self, weight_p, minweight=0.01,
                                zero_fermilevel=True):
        assert weight_p.dtype == float
        u = 0  # TODO

        absweight_p = np.absolute(weight_p)
        tot_weight = weight_p.sum()
        rest_weight = tot_weight
        eig_n = self.eig_un[u].copy()
        if zero_fermilevel:
            eig_n -= self.fermilevel

        txt = ''
        txt += ('# %6s %3s(%8s)    %3s(%8s)  %12s %14s\n' %
                ('p', 'i', 'eV', 'a', 'eV', 'Ediff (eV)', 'weight'))
        p_s = np.argsort(absweight_p)[::-1]
        for s, p in enumerate(p_s):
            i, a = self.ia_p[p]
            if absweight_p[p] < minweight:
                break
            txt += ('  %6s %3d(%8.3f) -> %3d(%8.3f): %12.4f %+14.4f\n' %
                    (p, i, eig_n[i] * Hartree, a, eig_n[a] * Hartree,
                     self.w_p[p] * Hartree, weight_p[p]))
            rest_weight -= weight_p[p]
        txt += ('  %37s: %12s %+14.4f\n' %
                ('rest', '', rest_weight))
        txt += ('  %37s: %12s %+14.4f\n' %
                ('total', '', tot_weight))
        return txt

    def plot_TCM(self, weight_p,
                 occ_energy_min, occ_energy_max,
                 unocc_energy_min, unocc_energy_max,
                 delta_energy, sigma, zero_fermilevel=True,
                 spectrum=False,
                 vmax='80%'):
        import matplotlib.pyplot as plt
        from matplotlib.gridspec import GridSpec

        # Calculate TCM
        args = (weight_p,
                occ_energy_min, occ_energy_max,
                unocc_energy_min, unocc_energy_max,
                delta_energy, sigma, zero_fermilevel)
        r = self.get_TCM(*args)
        energy_o, energy_u, dos_o, dos_u, tcm_ou, fermilevel = r

        # Start plotting
        plt.figure(figsize=(8, 8))
        linecolor = 'k'

        # Generate axis
        def get_gs(**kwargs):
            width = 0.84
            bottom = 0.12
            left = 0.12
            return GridSpec(2, 2, width_ratios=[3, 1], height_ratios=[1, 3],
                            bottom=bottom, top=bottom + width,
                            left=left, right=left + width,
                            **kwargs)
        gs = get_gs(hspace=0.05, wspace=0.05)
        ax_occ_dos = plt.subplot(gs[0])
        ax_unocc_dos = plt.subplot(gs[3])
        ax_tcm = plt.subplot(gs[2])
        if spectrum:
            ax_spec = plt.subplot(get_gs(hspace=0.8, wspace=0.8)[1])
        else:
            ax_spec = None

        # Plot TCM
        ax = ax_tcm
        plt.sca(ax)
        if isinstance(vmax, str):
            assert vmax[-1] == '%'
            tcmmax = max(np.max(tcm_ou), -np.min(tcm_ou))
            vmax = tcmmax * float(vmax[:-1]) / 100.0
        vmin = -vmax
        cmap = 'seismic'
        plt.pcolormesh(energy_o, energy_u, tcm_ou.T,
                       cmap=cmap, rasterized=True, vmin=vmin, vmax=vmax)
        plt.axhline(fermilevel, c=linecolor)
        plt.axvline(fermilevel, c=linecolor)

        ax.tick_params(axis='both', which='major', pad=2)
        plt.xlabel(r'Occ. energy $\varepsilon_{o}$ (eV)', labelpad=0)
        plt.ylabel(r'Unocc. energy $\varepsilon_{u}$ (eV)', labelpad=0)
        plt.xlim(occ_energy_min, occ_energy_max)
        plt.ylim(unocc_energy_min, unocc_energy_max)

        # Plot DOSes
        def plot_DOS(ax, energy_e, dos_e,
                     energy_min, energy_max,
                     dos_min, dos_max,
                     flip=False):
            ax.xaxis.set_ticklabels([])
            ax.yaxis.set_ticklabels([])
            ax.spines['right'].set_visible(False)
            ax.spines['top'].set_visible(False)
            ax.yaxis.set_ticks_position('left')
            ax.xaxis.set_ticks_position('bottom')
            if flip:
                set_label = ax.set_xlabel
                fill_between = ax.fill_betweenx
                set_energy_lim = ax.set_ylim
                set_dos_lim = ax.set_xlim

                def plot(x, y, *args, **kwargs):
                    return ax.plot(y, x, *args, **kwargs)
            else:
                set_label = ax.set_ylabel
                fill_between = ax.fill_between
                set_energy_lim = ax.set_xlim
                set_dos_lim = ax.set_ylim

                def plot(x, y, *args, **kwargs):
                    return ax.plot(x, y, *args, **kwargs)
            fill_between(energy_e, 0, dos_e, color='0.8')
            plot(energy_e, dos_e, 'k')
            set_label('DOS', labelpad=0)
            set_energy_lim(energy_min, energy_max)
            set_dos_lim(dos_min, dos_max)

        dos_min = 0.0
        dos_max = max(np.max(dos_o), np.max(dos_u))
        plot_DOS(ax_occ_dos, energy_o, dos_o,
                 occ_energy_min, occ_energy_max,
                 dos_min, dos_max,
                 flip=False)
        plot_DOS(ax_unocc_dos, energy_u, dos_u,
                 unocc_energy_min, unocc_energy_max,
                 dos_min, dos_max,
                 flip=True)

        return ax_tcm, ax_occ_dos, ax_unocc_dos, ax_spec

    def get_TCM(self, weight_p,
                occ_energy_min, occ_energy_max,
                unocc_energy_min, unocc_energy_max,
                delta_energy, sigma, zero_fermilevel):
        assert weight_p.dtype == float
        u = 0  # TODO

        eig_n = self.eig_un[u].copy()
        if zero_fermilevel:
            eig_n -= self.fermilevel
            fermilevel = 0.0
        else:
            fermilevel = self.fermilevel

        eig_n *= Hartree
        fermilevel *= Hartree

        # Inclusive arange function
        def arange(x0, x1, dx):
            return np.arange(x0, x1 + 0.5 * dx, dx)

        energy_o = arange(occ_energy_min, occ_energy_max, delta_energy)
        energy_u = arange(unocc_energy_min, unocc_energy_max, delta_energy)

        def gauss_ne(energy_e):
            energy_ne = energy_e[np.newaxis, :]
            eig_ne = eig_n[:, np.newaxis]
            norm = 1.0 / (sigma * np.sqrt(2 * np.pi))
            return norm * np.exp(-0.5 * (energy_ne - eig_ne)**2 / sigma**2)

        G_no = gauss_ne(energy_o)
        G_nu = gauss_ne(energy_u)

        # DOS
        dos_o = 2.0 * np.sum(G_no, axis=0)
        dos_u = 2.0 * np.sum(G_nu, axis=0)
        dosmax = max(np.max(dos_o), np.max(dos_u))
        dos_o /= dosmax
        dos_u /= dosmax

        def is_between(x, xmin, xmax):
            return xmin <= x and x <= xmax

        flt_p = []
        buf = 4 * sigma
        for p, weight in enumerate(weight_p):
            i, a = self.ia_p[p]
            if (is_between(eig_n[i],
                           occ_energy_min - buf,
                           occ_energy_max + buf) and
                is_between(eig_n[a],
                           unocc_energy_min - buf,
                           unocc_energy_max + buf)):
                flt_p.append(p)

        weight_f = weight_p[flt_p]
        G_fo = G_no[self.ia_p[flt_p, 0]]
        G_fu = G_nu[self.ia_p[flt_p, 1]]
        G_of = G_fo.T
        tcm_ou = np.dot(G_of * weight_f, G_fu)

        return energy_o, energy_u, dos_o, dos_u, tcm_ou, fermilevel



from ase import Atom, Atoms
from ase.parallel import parprint

from gpaw import GPAW, mpi
from gpaw.test import equal
from gpaw.lrtddft import LrTDDFT
from gpaw.poisson import FDPoissonSolver
from gpaw.pes.dos import DOSPES
from gpaw.pes.tddft import TDDFTPES

txt = None
R=0.7 # approx. experimental bond length
a = 3.0
c = 3.0
h = .3
H2 = Atoms([Atom('H', (a / 2, a / 2, (c - R) / 2)),
                Atom('H', (a / 2, a / 2, (c + R) / 2))],
               cell=(a, a, c))

H2_plus = Atoms([Atom('H', (a / 2, a / 2, (c - R) / 2)),
                Atom('H', (a / 2, a / 2, (c + R) / 2))],
               cell=(a, a, c))

xc='LDA'

calc = GPAW(gpts=(12, 12, 12), xc=xc, nbands=1, poissonsolver=FDPoissonSolver(),
            parallel={'domain': mpi.world.size},
            spinpol=True, txt=txt)
H2.set_calculator(calc)
e_H2 = H2.get_potential_energy()
niter_H2 = calc.get_number_of_iterations()


calc_plus = GPAW(gpts=(12, 12, 12), xc=xc, nbands=2, poissonsolver=FDPoissonSolver(),
                 parallel={'domain': mpi.world.size},
                 spinpol=True, txt=txt)
calc_plus.set(charge=+1)
H2_plus.set_calculator(calc_plus)
e_H2_plus = H2_plus.get_potential_energy()
niter_H2_plus = calc.get_number_of_iterations()

out = 'dospes.dat'
pes = DOSPES(calc, calc_plus, shift=True)
pes.save_folded_pes(filename=out, folding=None)
parprint('DOS:')
pes.save_folded_pes(filename=None, folding=None)

# check for correct shift
VDE = calc_plus.get_potential_energy() - calc.get_potential_energy()
BE_HOMO = 1.e23
be_n, f_n = pes.get_energies_and_weights()
for be, f in zip(be_n, f_n):
    if f > 0.1 and be < BE_HOMO:
        BE_HOMO = be
equal(BE_HOMO, VDE)

lr = LrTDDFT(calc_plus, xc=xc)

out = 'lrpes.dat'
pes = TDDFTPES(calc, lr)
pes.save_folded_pes(filename=out, folding='Gauss')
parprint('Linear response:')
pes.save_folded_pes(filename=None, folding=None)

energy_tolerance = 0.001
niter_tolerance = 1
equal(e_H2, -3.90059, energy_tolerance)
equal(e_H2_plus, 10.5659703, energy_tolerance)

# io
out = 'lrpes.dat.gz'
lr.write(out)
lr = LrTDDFT(out)
lr.set_calculator(calc_plus)
pes = TDDFTPES(calc, lr)
parprint('Linear response:')
pes.save_folded_pes(filename=None, folding=None)



from gpaw.lcaotddft.observer import TDDFTObserver


class RestartDumper(TDDFTObserver):
    def __init__(self, paw, restart_filename, interval=100):
        TDDFTObserver.__init__(self, paw, interval)
        self.restart_filename = restart_filename

    def _update(self, paw):
        if paw.niter == 0:
            return
        paw.log('%s activated' % self.__class__.__name__)
        for obs, n, args, kwargs in paw.observers:
            if (isinstance(obs, TDDFTObserver) and
                hasattr(obs, 'write_restart')):
                obs.write_restart()
        paw.write(self.restart_filename, mode='all')



"""Test the calculation of the excitation energy of Na2 by RSF and IVOs."""
from ase import Atoms
from ase.units import Hartree
from gpaw import GPAW, setup_paths
from gpaw.mpi import world
from gpaw.occupations import FermiDirac
from gpaw.test import equal, gen
from gpaw.eigensolvers import RMMDIIS
from gpaw.cluster import Cluster
from gpaw.lrtddft import LrTDDFT
import _gpaw

libxc_version = getattr(_gpaw, 'libxc_version', '2.x.y')
if int(libxc_version.split('.')[0]) < 3:
    from unittest import SkipTest
    raise SkipTest

h = 0.35  # Gridspacing
e_singlet = 4.61
e_singlet_lr = 5.54

if setup_paths[0] != '.':
    setup_paths.insert(0, '.')

gen('Mg', xcname='PBE', scalarrel=True, exx=True, yukawa_gamma=0.38)

c = {'energy': 0.05, 'eigenstates': 3, 'density': 3}
na2 = Cluster(Atoms('Mg', positions=[[0,0,0]]))
na2.minimal_box(2.5, h=h)
calc = GPAW(txt='mg_ivo.txt', xc='LCY-PBE:omega=0.38:excitation=singlet',
            eigensolver=RMMDIIS(), h=h, occupations=FermiDirac(width=0.0),
            spinpol=False, convergence=c)
na2.set_calculator(calc)
na2.get_potential_energy()
(eps_homo, eps_lumo) = calc.get_homo_lumo()
e_ex = eps_lumo - eps_homo
equal(e_singlet, e_ex, 0.15)
calc.write('mg.gpw')
c2 = GPAW('mg.gpw')
assert c2.hamiltonian.xc.excitation == 'singlet'
lr = LrTDDFT(calc, txt='LCY_TDDFT_Mg.log', istart=4, jend=5, nspins=2)
lr.write('LCY_TDDFT_Mg.ex.gz')
if world.rank == 0:
    lr2 = LrTDDFT('LCY_TDDFT_Mg.ex.gz')
    lr2.diagonalize()
    ex_lr = lr2[1].get_energy() * Hartree
    equal(e_singlet_lr, ex_lr, 0.15)





import numpy as np
from ase.build import molecule
from ase.parallel import parprint
from gpaw import GPAW
from gpaw.elf import ELF
from gpaw.test import equal
from gpaw.mpi import rank, world

atoms = molecule('CO')
atoms.center(2.0)

txt = '-'
# txt = None

calc = GPAW(h=0.24, txt=txt)
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()

elf = ELF(calc)
elf.update()
elf_G = elf.get_electronic_localization_function(gridrefinement=1)
elf_g = elf.get_electronic_localization_function(gridrefinement=2)

nt_G = calc.density.nt_sG[0]
taut_G = elf.taut_sG[0]
nt_grad2_G = elf.nt_grad2_sG[0]
nt_grad2_g = elf.nt_grad2_sg[0]

# integrate the CO bond
if rank == 0:
    # bond area
    x0 = (atoms.positions[0][0] - 1.0) / atoms.get_cell()[0, 0]
    x1 = 1 - x0
    y0 = (atoms.positions[0][1] - 1.0) / atoms.get_cell()[1, 1]
    y1 = 1 - y0
    z0 = atoms.positions[1][2] / atoms.get_cell()[2, 2]
    z1 = atoms.positions[0][2] / atoms.get_cell()[2, 2]
    gd = calc.wfs.gd
    Gx0, Gx1 = int(gd.N_c[0] * x0), int(gd.N_c[0] * x1)
    Gy0, Gy1 = int(gd.N_c[1] * y0), int(gd.N_c[1] * y1)
    Gz0, Gz1 = int(gd.N_c[2] * z0), int(gd.N_c[2] * z1)
    finegd = calc.density.finegd
    gx0, gx1 = int(finegd.N_c[0] * x0), int(finegd.N_c[0] * x1)
    gy0, gy1 = int(finegd.N_c[1] * y0), int(finegd.N_c[1] * y1)
    gz0, gz1 = int(finegd.N_c[2] * z0), int(finegd.N_c[2] * z1)
    int1 = elf_G[Gx0:Gx1, Gy0:Gy1, Gz0:Gz1].sum() * gd.dv
    int2 = elf_g[gx0:gx1, gy0:gy1, gz0:gz1].sum() * finegd.dv
    parprint("Ints", int1, int2)
    parprint("Min, max G", np.min(elf_G), np.max(elf_G))
    parprint("Min, max g", np.min(elf_g), np.max(elf_g))
#   The tested values (< r7887) do not seem to be correct
#    equal(int1, 14.8078, 0.0001)
#    equal(int2, 13.0331, 0.0001)

# check spin-polarized version
calc = GPAW(h=0.24,
            txt=txt,
            spinpol=True,
            parallel={'domain': world.size})
atoms.set_calculator(calc)
energy_spinpol = atoms.get_potential_energy()

    
def check_diff(g1, g2, gd, txt):
    # print rank, txt, "shapes", g1.shape, g2.shape
    intd = gd.integrate(np.abs(g1 - g2))
    parprint(txt, 'integrated diff=', intd, end='')
    maxd = np.max(np.abs(g1 - g2))
    parprint('max diff=', maxd)
    equal(intd, 0, 1e-8)
    equal(maxd, 0, 1.e-9)

nt_spinpol_G = calc.density.nt_sG.sum(axis=0)
check_diff(nt_G, nt_spinpol_G, elf.finegd, 'nt_G')
   
equal(energy, energy_spinpol, 0.0001)

elf_spinpol = ELF(calc)
elf_spinpol.update()
elf_spinpol_G = elf_spinpol.get_electronic_localization_function(
    gridrefinement=1)
elf_spinpol_g = elf_spinpol.get_electronic_localization_function(
    gridrefinement=2)
taut_spinpol_G = elf_spinpol.taut_sG.sum(axis=0)
check_diff(taut_G, taut_spinpol_G, elf.gd, 'taut_G')

nt_grad2_spinpol_G = 2 * elf_spinpol.nt_grad2_sG.sum(axis=0)
check_diff(nt_grad2_G, nt_grad2_spinpol_G, elf.gd, 'nt_grad2_G')

nt_grad2_spinpol_g = 2 * elf_spinpol.nt_grad2_sg.sum(axis=0)
check_diff(nt_grad2_g, nt_grad2_spinpol_g, elf.finegd, 'nt_grad2_g')

check_diff(elf_G, elf_spinpol_G, elf.gd, 'elf_G')
check_diff(elf_g, elf_spinpol_g, elf.finegd, 'elf_g')



import numpy as np
from gpaw.xc.libxc import LibXC, short_names
from gpaw.xc.kernel import XCKernel, codes
from gpaw.test import equal

funcs = []
modes = []
for name in short_names:
    try:
        LibXC(name)
    except NameError:
        continue
    if name == 'SCAN':
        continue
    funcs.append(name)
    modes.append(0)
for name in codes:
    funcs.append(name)
    modes.append(1)


def create_xc(func, mode):
    if mode == 0:
        xc = LibXC(func)
    else:
        xc = XCKernel(func)
    return xc


def f1(n_xg, xc):
    e_g = np.empty_like(n_xg[0])
    n_sg = n_xg[:1]
    sigma_xg = n_xg[1:2]
    tau_sg = n_xg[2:]
    dedn_sg = np.zeros_like(n_sg)
    dedsigma_xg = np.zeros_like(sigma_xg)
    dedtau_sg = np.zeros_like(tau_sg)
    xc.calculate(e_g, n_sg, dedn_sg, sigma_xg, dedsigma_xg, tau_sg, dedtau_sg)
    return e_g, np.concatenate((dedn_sg, dedsigma_xg, dedtau_sg))


def f2(n_xg, xc):
    e_g = np.empty_like(n_xg[0])
    n_sg = n_xg[:2]
    sigma_xg = n_xg[2:5]
    tau_sg = n_xg[5:]
    dedn_sg = np.zeros_like(n_sg)
    dedsigma_xg = np.zeros_like(sigma_xg)
    dedtau_sg = np.zeros_like(tau_sg)
    xc.calculate(e_g, n_sg, dedn_sg, sigma_xg, dedsigma_xg, tau_sg, dedtau_sg)
    return e_g, np.concatenate((dedn_sg, dedsigma_xg, dedtau_sg))

eps = 1.0e-6

n_xg = np.array(
    [[0.2, 0.01, 0.4],
     [0.2, 0.1, 0.5],
     [0.01, 0.01, 0.2],
     [0.1, 0.3, 0.5]]).T.copy()

for i, func in enumerate(funcs):
    print(i, func)
    xc = create_xc(funcs[i], modes[i])
    e0_g, d0_xg = f1(n_xg, xc)
    d_xg = np.empty_like(d0_xg)
    for x, n_g in enumerate(n_xg):
        m_xg = n_xg.copy()
        m_xg[x] += eps
        d_xg[x] = 0.5 * f1(m_xg, xc)[0] / eps
        m_xg[x] -= 2 * eps
        d_xg[x] -= 0.5 * f1(m_xg, xc)[0] / eps

    ns_xg = np.empty((7, len(n_g)))
    ns_xg[:2] = n_xg[0] / 2
    ns_xg[2:5] = n_xg[1] / 4
    ns_xg[5:] = n_xg[2] / 2
    es_g, ds_xg = f2(ns_xg, xc)
    error = (abs(d0_xg - d_xg).max() +
             abs(es_g - e0_g).max() +
             abs(ds_xg[:2] - d0_xg[0]).max() +
             abs(ds_xg[2:5].sum(0) / 4 - d0_xg[1]).max() +
             abs(ds_xg[5:] - d0_xg[2]).max())
    equal(error, 0, 6e-9)
    del xc

# Numbers from old lxc_xc.py test:
na = 2.0
nb = 1.0
sigma0 = 2.0  # (0.0, 1.0, 1.0)
sigma1 = 2.0
sigma2 = 5.0  # (1.0, 2.0, 0.0)
taua = (3 * np.pi**2)**(2. / 3.) * na**(5. / 3.) / 2 * sigma0
taub = (3 * np.pi**2)**(2. / 3.) * nb**(5. / 3.) / 2 * sigma2

n_xg = np.array(
    [[na, nb, sigma0, sigma1, sigma2, taua, taub],
     [0.1, 0.1, 0.025, 0.025, 0.025, 0.25, 0.25],
     [0.1, 0.1, 0.125, 0.125, 0.125, 0.0025, 0.025],
     [0.1, 0.1, 0.01, 0.01, 0.01, 0.2, 0.2],
     [0.1, 0.2, 0.1, -0.08, 0.10, 0.01, 0.05],
     [0.1, 0.1, 0.1, 0.01, 0.01, 0.01, 0.01],
     [0.1, 0.1, 0.1, 0.15, 0.20, 0.01, 0.05]]).T.copy()


for i, func in enumerate(funcs):
    xc = create_xc(funcs[i], modes[i])
    if xc.type == 'MGGA':
        N_xg = n_xg[:, :1].copy()
    else:
        N_xg = n_xg
    e0_g, d0_xg = f2(N_xg, xc)
    d_xg = np.empty_like(d0_xg)
    for x, n_g in enumerate(N_xg):
        m_xg = N_xg.copy()
        m_xg[x] += eps
        d_xg[x] = 0.5 * f2(m_xg, xc)[0] / eps
        m_xg[x] -= 2 * eps
        d_xg[x] -= 0.5 * f2(m_xg, xc)[0] / eps
    equal(abs(d0_xg - d_xg).max(), 0, 2e-8)
    del xc



import re
import numpy as np

from gpaw.lcaotddft.observer import TDDFTObserver


def convert_repr(r):
    # Integer
    try:
        return int(r)
    except ValueError:
        pass
    # Boolean
    b = {repr(False): False, repr(True): True}.get(r, None)
    if b is not None:
        return b
    # String
    s = r[1:-1]
    if repr(s) == r:
        return s
    raise RuntimeError('Unknown value: %s' % r)


class DipoleMomentWriter(TDDFTObserver):
    version = 1

    def __init__(self, paw, filename, center=False, density='comp',
                 interval=1):
        TDDFTObserver.__init__(self, paw, interval)
        self.master = paw.world.rank == 0
        if paw.niter == 0:
            # Initialize
            self.do_center = center
            self.density_type = density
            if self.master:
                self.fd = open(filename, 'w')
        else:
            # Read and continue
            self.read_header(filename)
            if self.master:
                self.fd = open(filename, 'a')

    def _write(self, line):
        if self.master:
            self.fd.write(line)
            self.fd.flush()

    def _write_header(self, paw):
        if paw.niter != 0:
            return
        line = '# %s[version=%s]' % (self.__class__.__name__, self.version)
        line += ('(center=%s, density=%s)\n' %
                 (repr(self.do_center), repr(self.density_type)))
        line += ('# %15s %15s %22s %22s %22s\n' %
                 ('time', 'norm', 'dmx', 'dmy', 'dmz'))
        self._write(line)

    def read_header(self, filename):
        with open(filename, 'r') as f:
            line = f.readline()
        m_i = re.split("[^a-zA-Z0-9_=']+", line[2:])
        assert m_i.pop(0) == self.__class__.__name__
        for m in m_i:
            if '=' not in m:
                continue
            k, v = m.split('=')
            v = convert_repr(v)
            if k == 'version':
                assert v == self.version
                continue
            # Translate key
            k = {'center': 'do_center', 'density': 'density_type'}[k]
            setattr(self, k, v)

    def _write_kick(self, paw):
        time = paw.time
        kick = paw.kick_strength
        line = '# Kick = [%22.12le, %22.12le, %22.12le]; ' % tuple(kick)
        line += 'Time = %.8lf\n' % time
        self._write(line)

    def calculate_dipole_moment(self, gd, rho_g, center=True):
        if center:
            center_v = 0.5 * gd.cell_cv.sum(0)
        else:
            center_v = np.zeros(3, dtype=float)
        r_vg = gd.get_grid_point_coordinates()
        dm_v = np.zeros(3, dtype=float)
        for v in range(3):
            dm_v[v] = - gd.integrate((r_vg[v] - center_v[v]) * rho_g)
        return dm_v

    def _write_dm(self, paw):
        time = paw.time
        density = paw.density
        if self.density_type == 'comp':
            rho_g = density.rhot_g
            gd = density.finegd
        elif self.density_type == 'pseudo':
            rho_g = density.nt_sg.sum(axis=0)
            gd = density.finegd
        elif self.density_type == 'pseudocoarse':
            rho_g = density.nt_sG.sum(axis=0)
            gd = density.gd
        else:
            raise RuntimeError('Unknown density type: %s' % self.density_type)

        norm = gd.integrate(rho_g)
        # dm = self.calculate_dipole_moment(gd, rho_g, center=self.do_center)
        dm = gd.calculate_dipole_moment(rho_g, center=self.do_center)
        line = ('%20.8lf %20.8le %22.12le %22.12le %22.12le\n' %
                (time, norm, dm[0], dm[1], dm[2]))
        self._write(line)

    def _update(self, paw):
        if paw.action == 'init':
            self._write_header(paw)
        elif paw.action == 'kick':
            self._write_kick(paw)
        self._write_dm(paw)

    def __del__(self):
        if self.master:
            self.fd.close()
        TDDFTObserver.__del__(self)



import numpy as np
from ase.units import Bohr
from ase.build import bulk
from gpaw import GPAW, FermiDirac
from gpaw.response.df import DielectricFunction
from gpaw.test import equal, findpeak

a = 6.75 * Bohr
atoms = bulk('C', 'diamond', a=a)

calc = GPAW(mode='pw',
            kpts=(3, 3, 3),
            eigensolver='rmm-diis',
            occupations=FermiDirac(0.001))

atoms.set_calculator(calc)
atoms.get_potential_energy()
calc.write('C.gpw', 'all')

eM1_ = 9.727
eM2_ = 9.548
w0_ = 10.7782
I0_ = 5.47
w_ = 10.7532
I_ = 5.98

# Macroscopic dielectric constant calculation
df = DielectricFunction('C.gpw', frequencies=(0.,), eta=0.001, ecut=50,
                        hilbert=False)
eM1, eM2 = df.get_macroscopic_dielectric_constant()
equal(eM1, eM1_, 0.01)
equal(eM2, eM2_, 0.01)

# Absorption spectrum calculation RPA
df = DielectricFunction('C.gpw', eta=0.25, ecut=50,
                        frequencies=np.linspace(0, 24., 241), hilbert=False)
a0, a = df.get_dielectric_function(filename=None)
df.check_sum_rule(a.imag)

equal(a0[0].real, eM1_, 0.01)
equal(a[0].real, eM2_, 0.01)
w, I = findpeak(np.linspace(0, 24., 241), a0.imag)
equal(w, w0_, 0.01)
equal(I / (4 * np.pi), I0_, 0.1)
w, I = findpeak(np.linspace(0, 24., 241), a.imag)
equal(w, w_, 0.01)
equal(I / (4 * np.pi), I_, 0.1)

a0, a = df.get_polarizability(filename=None)

w, I = findpeak(np.linspace(0, 24., 241), a0.imag)
equal(w, w0_, 0.01)
equal(I, I0_, 0.01)
w, I = findpeak(np.linspace(0, 24., 241), a.imag)
equal(w, w_, 0.01)
equal(I, I_, 0.01)

# Absorption spectrum calculation ALDA
w0_ = 10.7931
I0_ = 5.36
w_ = 10.7562
I_ = 5.8803

a0, a = df.get_polarizability(filename=None, xc='ALDA')

w, I = findpeak(np.linspace(0, 24., 241), a0.imag)
equal(w, w0_, 0.01)
equal(I, I0_, 0.1)
w, I = findpeak(np.linspace(0, 24., 241), a.imag)
equal(w, w_, 0.01)
equal(I, I_, 0.1)

# Absorption spectrum calculation long-range kernel
w0_ = 10.2189
I0_ = 5.14
w_ = 10.2906
I_ = 5.6955

a0, a = df.get_polarizability(filename=None, xc='LR0.25')

w, I = findpeak(np.linspace(0, 24., 241), a0.imag)
equal(w, w0_, 0.01)
equal(I, I0_, 0.1)
w, I = findpeak(np.linspace(0, 24., 241), a.imag)
equal(w, w_, 0.01)
equal(I, I_, 0.1)

# Absorption spectrum calculation Bootstrap
w0_ = 10.37
I0_ = 5.27
w_ = 10.4600
I_ = 6.0263

a0, a = df.get_polarizability(filename=None, xc='Bootstrap')

w, I = findpeak(np.linspace(0, 24., 241), a0.imag)
equal(w, w0_, 0.02)
equal(I, I0_, 0.2)
w, I = findpeak(np.linspace(0, 24., 241), a.imag)
equal(w, w_, 0.02)
equal(I, I_, 0.2)

# Absorption spectrum calculation RPA - Wigner-Seitz truncation
w0_ = 10.7780
I0_ = 5.5472
w_ = 10.7532
I_ = 6.0750

df_ws = DielectricFunction('C.gpw', eta=0.25, ecut=50,
                           frequencies=np.linspace(0, 24., 241), hilbert=False,
                           truncation='wigner-seitz')

a0_ws, a_ws = df_ws.get_polarizability(filename=None)

w, I = findpeak(np.linspace(0, 24., 241), a0_ws.imag)
equal(w, w0_, 0.02)
equal(I, I0_, 0.2)
w, I = findpeak(np.linspace(0, 24., 241), a_ws.imag)
equal(w, w_, 0.1)
equal(I, I_, 0.2)
# The Wigner-Seitz truncation does not give exactly the same for kpts=(3,3,3)



from gpaw.test import equal
from gpaw.grid_descriptor import GridDescriptor
from gpaw.spline import Spline
import gpaw.mpi as mpi
from gpaw.lfc import LocalizedFunctionsCollection as LFC

s = Spline(0, 1.0, [1.0, 0.5, 0.0])
n = 40
a = 8.0
gd = GridDescriptor((n, n, n), (a, a, a), comm=mpi.serial_comm)
c = LFC(gd, [[s], [s], [s]])
c.set_positions([(0.5, 0.5, 0.25 + 0.25 * i) for i in [0, 1, 2]])
b = gd.zeros()
c.add(b)
x = gd.integrate(b)

gd = GridDescriptor((n, n, n), (a, a, a), comm=mpi.serial_comm)
c = LFC(gd, [[s], [s], [s]])
c.set_positions([(0.5, 0.5, 0.25 + 0.25 * i) for i in [0, 1, 2]])
b = gd.zeros()
c.add(b)
y = gd.integrate(b)
equal(x, y, 1e-13)



import numpy as np

from gpaw.transformers import Transformer
from gpaw.fd_operators import Laplace

from gpaw.utilities.blas import axpy


class Preconditioner:
    def __init__(self, gd0, kin0, dtype=float, block=1):
        gd1 = gd0.coarsen()
        gd2 = gd1.coarsen()
        self.kin0 = kin0
        self.kin1 = Laplace(gd1, -0.5, 1, dtype)
        self.kin2 = Laplace(gd2, -0.5, 1, dtype)
        self.scratch0 = gd0.zeros((2, block), dtype, False)
        self.scratch1 = gd1.zeros((3, block), dtype, False)
        self.scratch2 = gd2.zeros((3, block), dtype, False)
        self.step = 0.66666666 / kin0.get_diagonal_element()

        self.restrictor_object0 = Transformer(gd0, gd1, 1, dtype)
        self.restrictor_object1 = Transformer(gd1, gd2, 1, dtype)
        self.interpolator_object2 = Transformer(gd2, gd1, 1, dtype)
        self.interpolator_object1 = Transformer(gd1, gd0, 1, dtype)
        self.restrictor0 = self.restrictor_object0.apply
        self.restrictor1 = self.restrictor_object1.apply
        self.interpolator2 = self.interpolator_object2.apply
        self.interpolator1 = self.interpolator_object1.apply

    def calculate_kinetic_energy(self, psit_xG, kpt):
        return None

    def __call__(self, residuals, kpt, ekin=None, out=None):
        if residuals.ndim == 3:
            if out is None:
                return self.__call__(residuals[np.newaxis], kpt)[0]
            return self.__call__(residuals[np.newaxis], kpt,
                                 out=out[np.newaxis])[0]
        nb = len(residuals)  # number of bands
        phases = kpt.phase_cd
        step = self.step
        if out is None:
            d0, q0 = self.scratch0[:, :nb]
        else:
            d0 = out
            q0 = self.scratch0[0, :nb]
        r1, d1, q1 = self.scratch1[:, :nb]
        r2, d2, q2 = self.scratch2[:, :nb]
        self.restrictor0(-residuals, r1, phases)
        d1[:] = 4 * step * r1
        self.kin1.apply(d1, q1, phases)
        q1 -= r1
        self.restrictor1(q1, r2, phases)
        d2 = 16 * step * r2
        self.kin2.apply(d2, q2, phases)
        q2 -= r2
        d2 -= 16 * step * q2
        self.interpolator2(d2, q1, phases)
        d1 -= q1
        self.kin1.apply(d1, q1, phases)
        q1 -= r1
        d1 -= 4 * step * q1
        self.interpolator1(-d1, d0, phases)
        self.kin0.apply(d0, q0, phases)
        q0 -= residuals
        axpy(-step, q0, d0)  # d0 -= step * q0
        d0 *= -1.0
        return d0



from ase import Atom
from gpaw import GPAW, MixerDif
from gpaw.cluster import Cluster
from gpaw.test import equal

h = .25
q = 0
box = 3.
spin=True

# B should not have spin contamination
s = Cluster([Atom('B')])
s.minimal_box(box, h=h)
s.set_initial_magnetic_moments([-1])

c = GPAW(xc='LDA', nbands=-3,
         basis='dzp',
         hund=True,
         charge=q, spinpol=spin, h=h,
         mixer=MixerDif(beta=0.05, nmaxold=5, weight=50.0),
         convergence={'eigenstates': 0.078, 'density': 5e-3, 'energy': 0.1},
         )
c.calculate(s)

contamination = min(c.density.get_spin_contamination(s, 0),
                    c.density.get_spin_contamination(s, 1))

equal(contamination, 0., 0.01)

# setup H2 at large distance with different spins for the atoms
s = Cluster([Atom('H'), Atom('H',[0,0,3.0])])
s.minimal_box(box, h=h)
s.set_initial_magnetic_moments([-1,1])

c = GPAW(xc='LDA', nbands=-3,
         charge=q, spinpol=spin, h=h,
         convergence={'eigenstates': 0.078, 'density': 1e-2, 'energy': 0.1},
         )
c.calculate(s)

scont_s = [c.density.get_spin_contamination(s),
           c.density.get_spin_contamination(s, 1)]
equal(scont_s[0], scont_s[1], 2.e-4) # symmetry
equal(scont_s[0], 0.9655, 1.5e-3)



"""Python wrapper for FFTW3 library."""

import os

import numpy as np

import _gpaw


ESTIMATE = 64
MEASURE = 0
PATIENT = 32
EXHAUSTIVE = 8


if os.environ.get('GPAW_FFTWSO'):
    import warnings
    warnings.warn('GPAW_FFTWSO is set to "{}"; ignoring.  '
                  'Please use customize.py to link FFTW instead.'
                  .format(os.environ['GPAW_FFTWSO']))


def have_fftw():
    return hasattr(_gpaw, 'FFTWPlan')


def check_fft_size(n):
    """Check if n is an efficient fft size.

    Efficient means that n can be factored into small primes (2, 3, 5, 7)."""

    if n == 1:
        return True
    for x in [2, 3, 5, 7]:
        if n % x == 0:
            return check_fft_size(n // x)
    return False


def get_efficient_fft_size(N, n=1):
    """Return smallest efficient fft size.

    Must be greater than or equal to N and divisible by n.
    """
    N = -(-N // n) * n
    while not check_fft_size(N):
        N += n
    return N


def check_fftw_inputs(in_R, out_R):
    for arr in in_R, out_R:
        # Note: Arrays not necessarily contiguous due to 16-byte alignment
        assert arr.ndim == 3  # We can perhaps relax this requirement
        assert arr.dtype == float or arr.dtype == complex

    if in_R.dtype == out_R.dtype == complex:
        assert in_R.shape == out_R.shape
    else:
        # One real and one complex:
        R, C = (in_R, out_R) if in_R.dtype == float else (out_R, in_R)
        assert C.dtype == complex
        assert R.shape[:2] == C.shape[:2]
        assert C.shape[2] == 1 + R.shape[2] // 2


class FFTWPlan:
    """FFTW3 3d transform."""
    def __init__(self, in_R, out_R, sign, flags=MEASURE):
        if not have_fftw():
            raise ImportError('Not compiled with FFTW.')

        check_fftw_inputs(in_R, out_R)

        self._ptr = _gpaw.FFTWPlan(in_R, out_R, sign, flags)
        self.in_R = in_R
        self.out_R = out_R
        self.sign = sign
        self.flags = flags

    def execute(self):
        _gpaw.FFTWExecute(self._ptr)

    def __del__(self):
        if getattr(self, '_ptr', None):
            _gpaw.FFTWDestroy(self._ptr)
        self._ptr = None


class NumpyFFTPlan:
    """Numpy fallback."""
    def __init__(self, in_R, out_R, sign, flags=None):
        check_fftw_inputs(in_R, out_R)
        self.in_R = in_R
        self.out_R = out_R
        self.sign = sign

    def execute(self):
        if self.in_R.dtype == float:
            self.out_R[:] = np.fft.rfftn(self.in_R)
        elif self.out_R.dtype == float:
            self.out_R[:] = np.fft.irfftn(self.in_R, self.out_R.shape)
            self.out_R *= self.out_R.size
        elif self.sign == 1:
            self.out_R[:] = np.fft.ifftn(self.in_R, self.out_R.shape)
            self.out_R *= self.out_R.size
        else:
            self.out_R[:] = np.fft.fftn(self.in_R)


def empty(shape, dtype=float):
    """numpy.empty() equivalent with 16 byte alignment."""
    assert dtype == complex
    N = np.prod(shape)
    a = np.empty(2 * N + 1)
    offset = (a.ctypes.data % 16) // 8
    a = a[offset:2 * N + offset].view(complex)
    a.shape = shape
    return a


if have_fftw():
    FFTPlan = FFTWPlan
else:
    FFTPlan = NumpyFFTPlan



from ase.build import molecule
from gpaw import GPAW
from gpaw.lcaotddft import LCAOTDDFT
from gpaw import KohnShamConvergenceError
from gpaw.utilities.timelimit import TimeLimiter

# Atoms
atoms = molecule('Na2')
atoms.center(vacuum=4.0)

# Ground-state calculation that will never converge
maxiter = 100
calc = GPAW(mode='lcao', basis='sz(dzp)', setups='1',
            convergence={'density': 1e-100},
            maxiter=maxiter)
atoms.set_calculator(calc)

tl = TimeLimiter(calc, timelimit=0, output='scf.txt')
tl.reset('scf', min_updates=3)
try:
    atoms.get_potential_energy()
except KohnShamConvergenceError:
    assert calc.scf.maxiter < maxiter, 'TimeLimiter did not break the SCF loop'
else:
    raise AssertionError('SCF loop ended too early')
calc.write('gs.gpw', mode='all')

# Time-propagation calculation that will never finish
td_calc = LCAOTDDFT('gs.gpw')
tl = TimeLimiter(td_calc, timelimit=0, output='tddft.txt')
tl.reset('tddft', min_updates=3)
td_calc.propagate(10, maxiter - td_calc.niter)
assert td_calc.maxiter < maxiter, 'TimeLimiter did not break the TDDFT loop'



import threading

import numpy as np


def run(task, nthreads, *args):
    """Parallelize task over first index of args."""
    
    N = len(args[0])

    def target(*args):
        for x in zip(*args):
            task(*x)

    if nthreads == 1:
        target(*args)
        return [0, N]
        
    n = np.linspace(0, N, nthreads + 1).round().astype(int)
    
    threads = []
    for n1, n2 in zip(n[:-1], n[1:]):
        thread = threading.Thread(target=target, args=[x[n1:n2] for x in args])
        thread.start()
        threads.append(thread)
    
    for thread in threads:
        thread.join()

    return n



from gpaw.cluster import Cluster
from ase.build import molecule
from ase.units import Pascal, m
from ase.data.vdw import vdw_radii
from gpaw.solvation import (
    SolvationGPAW,
    EffectivePotentialCavity,
    Power12Potential,
    LinearDielectric,
    GradientSurface,
    SurfaceInteraction,
)

h = 0.3
vac = 3.0
u0 = 0.180
epsinf = 80.
st = 18.4 * 1e-3 * Pascal * m
T = 298.15
vdw_radii = vdw_radii.copy()
vdw_radii[1] = 1.09
atomic_radii = lambda atoms: [vdw_radii[n] for n in atoms.numbers]
convergence = {
    'energy': 0.1 / 8.,
    'density': 10.,
    'eigenstates': 10.,
}

atoms = Cluster(molecule('H2O'))
atoms.minimal_box(vac, h)

calc = SolvationGPAW(
    xc='LDA', h=h, convergence=convergence,
    cavity=EffectivePotentialCavity(
        effective_potential=Power12Potential(atomic_radii, u0),
        temperature=T,
        surface_calculator=GradientSurface()
    ),
    dielectric=LinearDielectric(epsinf=epsinf),
    interactions=[SurfaceInteraction(surface_tension=st)]
)
atoms.calc = calc
atoms.get_potential_energy()
atoms.get_forces()
eps_gradeps = calc.hamiltonian.dielectric.eps_gradeps

# same molecules, different cell, reallocate
atoms = Cluster(molecule('H2O'))
atoms.positions[0][0] = atoms.positions[0][0] - 1.
atoms.minimal_box(vac, h)
atoms.calc = calc
atoms.get_potential_energy()
atoms.get_forces()
assert calc.hamiltonian.dielectric.eps_gradeps is not eps_gradeps
eps_gradeps = calc.hamiltonian.dielectric.eps_gradeps

# small position change, no reallocate
atoms.positions[0][0] = atoms.positions[0][0] + 1e-2
atoms.get_potential_energy()
atoms.get_forces()
assert calc.hamiltonian.dielectric.eps_gradeps is eps_gradeps
eps_gradeps = calc.hamiltonian.dielectric.eps_gradeps
radii = calc.hamiltonian.cavity.effective_potential.r12_a

# completely different atoms object, reallocate, read new radii
atoms = Cluster(molecule('NH3'))
atoms.minimal_box(vac, h)
atoms.calc = calc
atoms.get_potential_energy()
atoms.get_forces()
assert calc.hamiltonian.dielectric.eps_gradeps is not eps_gradeps
assert calc.hamiltonian.cavity.effective_potential.r12_a is not radii



from math import pi, sqrt

import numpy as np
from ase.units import Bohr, Ha

from gpaw.atom.shapefunc import shape_functions
from gpaw.fftw import get_efficient_fft_size
from gpaw.grid_descriptor import GridDescriptor
from gpaw.lfc import LFC
from gpaw.utilities import h2gpts
from gpaw.wavefunctions.pw import PWDescriptor
from gpaw.mpi import serial_comm


class Interpolator:
    def __init__(self, gd1, gd2, dtype=float):
        self.pd1 = PWDescriptor(0.0, gd1, dtype)
        self.pd2 = PWDescriptor(0.0, gd2, dtype)

    def interpolate(self, a_r):
        return self.pd1.interpolate(a_r, self.pd2)[0]


POINTS = 200


class PS2AE:
    """Transform PS to AE wave functions.

    Interpolates PS wave functions to a fine grid and adds PAW
    corrections in order to obtain true AE wave functions.
    """
    def __init__(self, calc, h=0.05, n=2):
        """Create transformation object.

        calc: GPAW calculator object
            The calcalator that has the wave functions.
        h: float
            Desired grid-spacing in Angstrom.
        n: int
            Force number of points to be a mulitiple of n.
        """
        self.calc = calc
        gd = calc.wfs.gd

        gd1 = GridDescriptor(gd.N_c, gd.cell_cv, comm=serial_comm)

        # Descriptor for the final grid:
        N_c = h2gpts(h / Bohr, gd.cell_cv)
        N_c = np.array([get_efficient_fft_size(N, n) for N in N_c])
        gd2 = self.gd = GridDescriptor(N_c, gd.cell_cv, comm=serial_comm)
        self.interpolator = Interpolator(gd1, gd2, self.calc.wfs.dtype)

        self.dphi = None  # PAW correction (will be initialized when needed)

    def _initialize_corrections(self):
        if self.dphi is not None:
            return
        splines = {}
        dphi_aj = []
        for setup in self.calc.wfs.setups:
            dphi_j = splines.get(setup)
            if dphi_j is None:
                rcut = max(setup.rcut_j) * 1.1
                gcut = setup.rgd.ceil(rcut)
                dphi_j = []
                for l, phi_g, phit_g in zip(setup.l_j,
                                            setup.data.phi_jg,
                                            setup.data.phit_jg):
                    dphi_g = (phi_g - phit_g)[:gcut]
                    dphi_j.append(setup.rgd.spline(dphi_g, rcut, l,
                                                   points=200))
            dphi_aj.append(dphi_j)

        self.dphi = LFC(self.gd, dphi_aj, kd=self.calc.wfs.kd.copy(),
                        dtype=self.calc.wfs.dtype)
        self.dphi.set_positions(self.calc.spos_ac)

    def get_wave_function(self, n, k=0, s=0, ae=True):
        """Interpolate wave function.

        n: int
            Band index.
        k: int
            K-point index.
        s: int
            Spin index.
        ae: bool
            Add PAW correction to get an all-electron wave function.
        """
        psi_r = self.calc.get_pseudo_wave_function(n, k, s,
                                                   pad=True, periodic=True)
        psi_R = self.interpolator.interpolate(psi_r * Bohr**1.5)
        if ae:
            self._initialize_corrections()
            wfs = self.calc.wfs
            P_nI = wfs.collect_projections(k, s)
            if wfs.world.rank == 0:
                P_ai = {}
                I1 = 0
                for a, setup in enumerate(wfs.setups):
                    I2 = I1 + setup.ni
                    P_ai[a] = P_nI[n, I1:I2]
                    I1 = I2
                self.dphi.add(psi_R, P_ai, k)
            wfs.world.broadcast(psi_R, 0)
        return psi_R * Bohr**-1.5

    def get_electrostatic_potential(self, ae=True, rcgauss=0.02):
        """Interpolate electrostatic potential.

        Return value in eV.

        ae: bool
            Add PAW correction to get the all-electron potential.
        rcgauss: float
            Width of gaussian (in Angstrom) used to represent the nuclear
            charge.
        """
        gd = self.calc.hamiltonian.finegd
        v_r = self.calc.get_electrostatic_potential() / Ha
        gd1 = GridDescriptor(gd.N_c, gd.cell_cv, comm=serial_comm)
        interpolator = Interpolator(gd1, self.gd)
        v_R = interpolator.interpolate(v_r)

        if ae:
            self.add_potential_correction(v_R, rcgauss / Bohr)

        return v_R * Ha

    def add_potential_correction(self, v_R, rcgauss: float) -> None:
        dens = self.calc.density
        dens.D_asp.redistribute(dens.atom_partition.as_serial())
        dens.Q_aL.redistribute(dens.atom_partition.as_serial())

        dv_a1 = []
        for a, D_sp in dens.D_asp.items():
            setup = dens.setups[a]
            c = setup.xc_correction
            rgd = c.rgd
            ghat_g = shape_functions(rgd,
                                     **setup.data.shape_function, lmax=0)[0]
            Z_g = shape_functions(rgd, 'gauss', rcgauss, lmax=0)[0] * setup.Z
            D_q = np.dot(D_sp.sum(0), c.B_pqL[:, :, 0])
            dn_g = np.dot(D_q, (c.n_qg - c.nt_qg)) * sqrt(4 * pi)
            dn_g += 4 * pi * (c.nc_g - c.nct_g)
            dn_g -= Z_g
            dn_g -= dens.Q_aL[a][0] * ghat_g * sqrt(4 * pi)
            dv_g = rgd.poisson(dn_g) / sqrt(4 * pi)
            dv_g[1:] /= rgd.r_g[1:]
            dv_g[0] = dv_g[1]
            dv_g[-1] = 0.0
            dv_a1.append([rgd.spline(dv_g, points=POINTS)])

        dens.D_asp.redistribute(dens.atom_partition)
        dens.Q_aL.redistribute(dens.atom_partition)

        if dv_a1:
            dv = LFC(self.gd, dv_a1)
            dv.set_positions(self.calc.spos_ac)
            dv.add(v_R)
        dens.gd.comm.broadcast(v_R, 0)


def interpolate_weight(calc, weight, h=0.05, n=2):
    """interpolates cdft weight function, gd is the fine grid."""
    gd = calc.density.finegd

    weight = gd.collect(weight, broadcast=True)
    weight = gd.zero_pad(weight)

    w = np.zeros_like(weight)
    gd1 = GridDescriptor(gd.N_c, gd.cell_cv, comm=serial_comm)
    gd1.distribute(weight, w)

    N_c = h2gpts(h / Bohr, gd.cell_cv)
    N_c = np.array([get_efficient_fft_size(N, n) for N in N_c])
    gd2 = GridDescriptor(N_c, gd.cell_cv, comm=serial_comm)

    interpolator = Interpolator(gd1, gd2)
    W = interpolator.interpolate(w)

    return W



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.tddft import TDDFT
from gpaw.poisson import PoissonSolver
from gpaw.mpi import world

from gpaw.test import equal

# Atoms
atoms = molecule('SiH4')
atoms.center(vacuum=4.0)

# Ground-state calculation
calc = GPAW(nbands=7, h=0.4,
            poissonsolver=PoissonSolver(eps=1e-16),
            convergence={'density': 1e-8},
            xc='GLLBSC',
            txt='gs.out')
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('gs.gpw', mode='all')

fxc = 'LDA'
# Time-propagation calculation with linearize_to_fxc()
td_calc = TDDFT('gs.gpw', txt='td.out')
td_calc.linearize_to_xc(fxc)
td_calc.absorption_kick(np.ones(3) * 1e-5)
td_calc.propagate(20, 4, 'dm.dat')
world.barrier()

# Test the absolute values
data = np.loadtxt('dm.dat').ravel()
if 0:
    from gpaw.test import print_reference
    print_reference(data, 'ref', '%.12le')

ref = [0.0, 1.46915686e-15, -3.289312570937e-14, -2.273046460905e-14,
       -3.201827522804e-15, 0.82682747, 1.41057108e-15, 6.113786981692e-05,
       6.113754003915e-05, 6.113827654045e-05, 1.65365493, 1.69317502e-16,
       0.0001066406796495, 0.0001066481954317, 0.0001066442404437, 2.4804824,
       -1.16927902e-15, 0.0001341112381952, 0.0001341042452292,
       0.0001341080813738]

print(data.tolist())

tol = 1e-7
equal(data, ref, tol)



from __future__ import print_function
import numpy as np
from math import pi
from gpaw.coulomb import Coulomb
from gpaw.grid_descriptor import GridDescriptor
from gpaw.mpi import world
from gpaw.utilities.gauss import coordinates
from gpaw.test import equal
import time

def test_coulomb(N=2**6, a=20):
    Nc = (N, N, N)            # Number of grid point
    gd = GridDescriptor(Nc, (a, a, a), True)  # grid-descriptor object
    xyz, r2 = coordinates(gd) # matrix with the square of the radial coordinate
    r = np.sqrt(r2)           # matrix with the values of the radial coordinate
    nH = np.exp(-2 * r) / pi  # density of the hydrogen atom
    C = Coulomb(gd)           # coulomb calculator
    
    if world.size > 1:
        C.load('real')
        t0 = time.time()
        print('Processor %s of %s: %s Ha in %s sec' % (
            gd.comm.rank + 1,
            gd.comm.size,
            -0.5 * C.coulomb(nH, method='real'),
            time.time() - t0))
        return
    else:
        C.load('recip_ewald')
        C.load('recip_gauss')
        C.load('real')
        test = {}
        t0 = time.time()
        test['dual density'] = (-0.5 * C.coulomb(nH, nH.copy()),
                                time.time() - t0)
        for method in ('real', 'recip_gauss', 'recip_ewald'):
            t0 = time.time()
            test[method] = (-0.5 * C.coulomb(nH, method=method),
                            time.time() - t0)
        return test

analytic = -5 / 16.0
res = test_coulomb(N=48, a=15)
if world.size == 1:
    print('Units: Bohr and Hartree')
    print('%12s %8s %8s' % ('Method', 'Energy', 'Time'))
    print('%12s %2.6f %6s' % ('analytic', analytic, '--'))
    for method, et in res.items():
        print('%12s %2.6f %1.7f' % ((method,) + et))

    equal(res['real'][0],         analytic, 6e-3)
    equal(res['recip_gauss'][0],  analytic, 6e-3)
    equal(res['recip_ewald'][0],  analytic, 2e-2)
    equal(res['dual density'][0], res['recip_gauss'][0], 1e-9)


# mpirun -np 2 python coulomb.py --gpaw-parallel --gpaw-debug



from __future__ import print_function
from ase import Atoms
from ase.units import Hartree
from gpaw import GPAW, PoissonSolver, FermiDirac, Davidson, MixerSum
from gpaw.test import equal

a = 5.0
n = 24
li = Atoms('Li', magmoms=[1.0], cell=(a, a, a), pbc=True)

calc = GPAW(gpts=(n, n, n), nbands=1, xc=dict(name='oldPBE', stencil=1),
            poissonsolver=PoissonSolver(),
            mixer=MixerSum(0.6, 5, 10.0),
            eigensolver=Davidson(4),
            convergence=dict(eigenstates=4.5e-8),
            occupations=FermiDirac(0.0))
li.set_calculator(calc)
e = li.get_potential_energy() + calc.get_reference_energy()
niter_PBE = calc.get_number_of_iterations()
equal(e, -7.462 * Hartree, 1.4)

calc.set(xc=dict(name='revPBE', stencil=1))
erev = li.get_potential_energy() + calc.get_reference_energy()
niter_revPBE = calc.get_number_of_iterations()

equal(erev, -7.487 * Hartree, 1.3)
equal(e - erev, 0.025 * Hartree, 0.002 * Hartree)

print(e, erev)
energy_tolerance = 0.002
niter_tolerance = 0
equal(e, -204.381098849, energy_tolerance)  # svnversion 5252
equal(erev, -205.012303379, energy_tolerance)  # svnversion 5252



from __future__ import print_function
from time import time

import numpy as np

n = 1000
a1 = np.eye(n)
a1 += 0.0001
a2 = 2 * a1

from gpaw.utilities.blas import gemm
b = np.zeros((n, n))
t0 = time()
gemm(1.0, a1, a2, 0.0, b)
tgpaw = time() - t0
print('gpaw.gemm  ', tgpaw) 

t0 = time()
c = np.dot(a1, a2)
tnumpy = time() - t0
print('numpy.dot  ', tnumpy)

"""
SLID:
gpaw.gemm   0.41842508316
numpy.dot   11.26800704

p019:
gpaw.gemm   0.444674015045
numpy.dot   11.8213479519

m022:
gpaw.gemm   0.446084022522
numpy.dot   11.9757530689

u091:
gpaw.gemm   0.377645015717
numpy.dot   0.389540910721

CASIMIR:
gpaw.gemm   2.19097900391
numpy.dot   8.21617603302

THUL:
gpaw.gemm   0.520259857178
numpy.dot   0.505489110947
"""



import time

class QuadraticETA:
    def __init__(self, min_times = 10):
        self.times = []
        self.min_times = max(min_times,3)

    def update(self):
        self.times.append(time.time())

    def eta(self, n):
        if ( len(self.times) < self.min_times ):
            #print '%6d' % (len(self.times))
            return 0.0

        ns = int(len(self.times) / 3.) 

        y1 = 0.0
        for i in range(len(self.times)-3*ns,len(self.times)-2*ns):
            y1 += self.times[i]
        y1 /= ns
        x1  = (len(self.times)-3*ns + len(self.times)-2*ns) / 2.

        y2 = 0.0
        for i in range(len(self.times)-2*ns,len(self.times)-ns):
            y2 += self.times[i]
        y2 /= ns
        x2  = (len(self.times)-2*ns + len(self.times)-1*ns) / 2.

        y3 = 0.0
        for i in range(len(self.times)-ns,len(self.times)):
            y3 += self.times[i]
        y3 /= ns
        x3  = (len(self.times)-1*ns + len(self.times)-0*ns) / 2.
        
        # 2nd order fit:
        #   y(x) = a * x**2 + b * x + c
        #   y'(x) = 2a * x + b
        #   y''(x) = 2a
        #   y2''(x2) = 2 a = [(y3-y2)/(x3-x2) - (y2-y1)/(x2-x1)] / [.5*(x3+x2)-.5*(x2+x1)]
        #   y2'(x2) = 2*a*x + b = (y3-y1)/(x3-x1)
        #   y2(x2) = a * x2**2 + b * x2 + c
        a = .5 * ( (y3-y2)/(x3-x2) - (y2-y1)/(x2-x1) ) / ( .5*(x3+x2)-.5*(x2+x1) )
        b = (y3-y1)/(x3-x1) - 2 * a * x2
        c = y2 - a * x2*x2 - b * x2

        #print x1,y1
        #print x2,y2
        #print x3,y3

        #print a
        #print b
        #print c

        #print '%6d %12.6lf %12.6lf %18.6lf %12.6lf %18.6lf' % (len(self.times), a, b, c-self.times[0], a * n*n + b * n + c - time.time(), a * n*n + b * n + c - self.times[0])

        return a * n*n + b * n + c - time.time()


if __name__ == "__main__":
    eta = QuadraticETA(10)

    N = 100

    print(time.time())
    t0 = time.time()
    for i in range(N):
        eta.update()
        eta.eta(N+1)
        for j in range(i+1):
            time.sleep(.001)

    t1 = time.time()
            
    print(time.time())
    
    print(t1-t0)



from __future__ import print_function
import numpy as np
from gpaw.lfc import LocalizedFunctionsCollection as LFC
from gpaw.grid_descriptor import GridDescriptor
from gpaw.spline import Spline
gd = GridDescriptor([20, 16, 16], [(4, 2, 0), (0, 4, 0), (0, 0, 4)])
spos_ac = np.array([[0.252, 0.15, 0.35], [0.503, 0.5, 0.5]])
s = Spline(l=0, rmax=2.0, f_g=np.array([1, 0.9, 0.1, 0.0]))
spline_aj = [[s], [s]]
c = LFC(gd, spline_aj)
c.set_positions(spos_ac)
c_ai = c.dict(zero=True)
if 1 in c_ai:
    c_ai[1][0] = 2.0
psi = gd.zeros()
c.add(psi, c_ai)

d_avv = dict([(a, np.zeros((3, 3))) for a in c.my_atom_indices])
c.second_derivative(psi, d_avv)

if 0 in d_avv:
    print(d_avv[0])

eps = 0.000001
d_aiv = c.dict(derivative=True)
pos_av = np.dot(spos_ac, gd.cell_cv)
for v in range(3):
    pos_av[0, v] += eps
    c.set_positions(np.dot(pos_av, gd.icell_cv.T))
    c.derivative(psi, d_aiv)
    if 0 in d_aiv:
        d0_v = d_aiv[0][0].copy()
    pos_av[0, v] -= 2 * eps
    c.set_positions(np.dot(pos_av, gd.icell_cv.T))
    c.derivative(psi, d_aiv)
    if 0 in d_aiv:
        d0_v -= d_aiv[0][0]
        d0_v /= -2 * eps
        print(d0_v)
        d_avv[0][v] -= d0_v
    pos_av[0, v] += eps
if 0 in d_avv:
    assert np.abs(d_avv[0]).max() < 1e-10

 

    



# Copyright (C) 2006 CSC-Scientific Computing Ltd.
# Please see the accompanying LICENSE file for further information.
from __future__ import print_function
import os
import platform
import sys
import re
import distutils.util
from distutils.sysconfig import get_config_vars
from distutils.version import LooseVersion
from glob import glob
from os.path import join
from pathlib import Path
from stat import ST_MTIME


def find_file(arg, dir, files):
    # looks if the first element of the list arg is contained in the list files
    # and if so, appends dir to to arg. To be used with the os.path.walk
    if arg[0] in files:
        arg.append(dir)


def get_system_config(define_macros, undef_macros,
                      include_dirs, libraries, library_dirs, extra_link_args,
                      extra_compile_args, runtime_library_dirs, extra_objects,
                      import_numpy):

    undef_macros += ['NDEBUG']
    if import_numpy:
        import numpy
        include_dirs += [numpy.get_include()]

    # libxc
    libraries += ['xc']

    machine = platform.uname()[4]
    if machine == 'sun4u':

        #  _
        # |_ | ||\ |
        #  _||_|| \|
        #

        extra_compile_args += ['-Kpic', '-fast']

        # Suppress warning from -fast (-xarch=native):
        f = open('cc-test.c', 'w')
        f.write('int main(){}\n')
        f.close()
        stderr = os.popen3('cc cc-test.c -fast')[2].read()
        arch = re.findall(r'-xarch=(\S+)', stderr)
        os.remove('cc-test.c')
        if len(arch) > 0:
            extra_compile_args += ['-xarch=%s' % arch[-1]]

        # We need the -Bstatic before the -lsunperf and -lfsu:
        # http://forum.java.sun.com/thread.jspa?threadID=5072537&messageID=9265782
        extra_link_args += ['-Bstatic', '-lsunperf', '-lfsu', '-Bdynamic']
        cc_version = os.popen3('cc -V')[2].readline().split()[3]
        if LooseVersion(cc_version) > '5.6':
            libraries.append('mtsk')
        else:
            extra_link_args.append('-lmtsk')
        # define_macros.append(('NO_C99_COMPLEX', '1'))

    elif sys.platform.startswith('win'):

        # We compile with mingw coming from pythonyx (32-bit)
        # on the msys command line, e.g.:
        # LIBRARY_PATH=/c/libxc/lib:/c/OpenBLAS/lib \
        # C_INCLUDE_PATH=/c/libxc/include python setup.py build
        if 'LIBRARY_PATH' in os.environ:
            library_dirs += os.environ['LIBRARY_PATH'].split(os.path.pathsep)

        extra_compile_args += ['-Wall', '-std=c99']

        lib = ''
        for ld in library_dirs:
            # OpenBLAS (includes Lapack)
            if os.path.exists(join(ld, 'libopenblas.a')):
                lib = 'openblas'
                break
        if lib == 'openblas':
            libraries += [lib, 'gfortran']

    elif sys.platform in ['aix5', 'aix6']:

        #
        # o|_  _ _
        # ||_)| | |
        #

        extra_compile_args += ['-qlanglvl=stdc99']
        # setting memory limit is necessary on aix5
        if sys.platform == 'aix5':
            extra_link_args += ['-bmaxdata:0x80000000',
                                '-bmaxstack:0x80000000']

        libraries += ['f', 'lapack', 'essl']
        define_macros.append(('GPAW_AIX', '1'))

    elif sys.platform == 'darwin':

        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']
        include_dirs += ['/usr/include/malloc']
        library_dirs += ['/usr/local/lib']   # libxc installed with Homebrew
        extra_link_args += ['-framework', 'Accelerate']  # BLAS

        # If the user uses python 3 from homebrew, then installation would
        # fail due to a broken LINKFORSHARED configuration variable
        cfgDict = get_config_vars()
        if ('LINKFORSHARED' in cfgDict and
            'Python.framework/Versions/3.6' in cfgDict['LINKFORSHARED']):
            cfgDict['LINKFORSHARED'] = '-l python3.6'

        # We should probably add something to allow for user-installed BLAS?

    elif machine in ['x86_64', 'ppc64', 'ppc64le', 'aarch64', 's390x']:

        #    _
        # \/|_||_    |_ |_|
        # /\|_||_| _ |_|  |
        #

        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']

        # Look for ACML libraries:
        acml = glob('/opt/acml*/g*64/lib')
        if len(acml) > 0:
            library_dirs += [acml[-1]]
            libraries += ['acml']
            if acml[-1].find('gfortran') != -1:
                libraries.append('gfortran')
            if acml[-1].find('gnu') != -1:
                libraries.append('g2c')
            extra_link_args += ['-Wl,-rpath=' + acml[-1]]
        else:
            atlas = False
            for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib64/atlas']:
                if glob(join(dir, 'libatlas.so')) != []:
                    atlas = True
                    libdir = dir
                    break
            satlas = False
            for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib64/atlas']:
                if glob(join(dir, 'libsatlas.so')) != []:
                    satlas = True
                    libdir = dir
                    break
            openblas = False
            for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib64']:
                if glob(join(dir, 'libopenblas.so')) != []:
                    openblas = True
                    libdir = dir
                    break
            if 'MKLROOT' in os.environ:
                libraries += ['mkl_intel_lp64', 'mkl_sequential', 'mkl_core',
                              'irc']
            elif openblas:  # prefer openblas
                libraries += ['openblas', 'lapack']
                library_dirs += [libdir]
            else:
                if atlas:  # then atlas
                    # http://math-atlas.sourceforge.net/errata.html#LINK
                    # atlas does not respect OMP_NUM_THREADS - build
                    # single-thread
                    # http://math-atlas.sourceforge.net/faq.html#tsafe
                    libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                    library_dirs += [libdir]
                elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                    libraries += ['satlas']
                    library_dirs += [libdir]
                else:
                    libraries += ['blas', 'lapack']

    elif machine == 'ia64':

        #  _  _
        # |_ |  o
        #  _||_||
        #

        extra_compile_args += ['-Wall', '-std=c99']
        libraries += ['mkl', 'mkl_lapack64']

    elif platform.machine().startswith('arm'):

        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']

        atlas = False
        for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
            if glob(join(dir, 'libatlas.so')) != []:
                atlas = True
                libdir = dir
                break
        satlas = False
        for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
            if glob(join(dir, 'libsatlas.so')) != []:
                satlas = True
                libdir = dir
                break
        openblas = False
        for dir in ['/usr/lib', '/usr/local/lib']:
            if glob(join(dir, 'libopenblas.so')) != []:
                openblas = True
                libdir = dir
                break
        if openblas:  # prefer openblas
            libraries += ['openblas']
            library_dirs += [libdir]
        else:
            if atlas:  # then atlas
                # http://math-atlas.sourceforge.net/errata.html#LINK
                # atlas does not respect OMP_NUM_THREADS - build single-thread
                # http://math-atlas.sourceforge.net/faq.html#tsafe
                libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                library_dirs += [libdir]
            elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                libraries += ['satlas']
                library_dirs += [libdir]
            else:
                libraries += ['blas', 'lapack']

    elif machine == 'i686':

        #      _
        # o|_ |_||_
        # ||_||_||_|
        #

        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']

        if 'MKL_ROOT' in os.environ:
            mklbasedir = [os.environ['MKL_ROOT']]
        else:
            mklbasedir = glob('/opt/intel/mkl*')
        libs = ['libmkl_ia32.a']
        if mklbasedir != []:
            os.path.walk(mklbasedir[0], find_file, libs)
        libs.pop(0)
        if libs != []:
            libs.sort()
            libraries += ['mkl_lapack',
                          'mkl_ia32', 'guide', 'pthread', 'mkl']
            library_dirs += libs
            # extra_link_args += ['-Wl,-rpath=' + library_dirs[-1]]
        else:
            atlas = False
            for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
                if glob(join(dir, 'libatlas.so')) != []:
                    atlas = True
                    libdir = dir
                    break
            satlas = False
            for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
                if glob(join(dir, 'libsatlas.so')) != []:
                    satlas = True
                    libdir = dir
                    break
            openblas = False
            for dir in ['/usr/lib', '/usr/local/lib']:
                if glob(join(dir, 'libopenblas.so')) != []:
                    openblas = True
                    libdir = dir
                    break
            if openblas:  # prefer openblas
                libraries += ['openblas']
                library_dirs += [libdir]
            else:
                if atlas:  # then atlas
                    # http://math-atlas.sourceforge.net/errata.html#LINK
                    # atlas does not respect OMP_NUM_THREADS - build
                    # single-thread
                    # http://math-atlas.sourceforge.net/faq.html#tsafe
                    libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                    library_dirs += [libdir]
                elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                    libraries += ['satlas']
                    library_dirs += [libdir]
                else:
                    libraries += ['blas', 'lapack']

            # add libg2c if available
            g2c = False
            for dir in ['/usr/lib', '/usr/local/lib']:
                if glob(join(dir, 'libg2c.so')) != []:
                    g2c = True
                    break
                if glob(join(dir, 'libg2c.a')) != []:
                    g2c = True
                    break
            if g2c:
                libraries += ['g2c']

    else:
        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']

        atlas = False
        for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
            if glob(join(dir, 'libatlas.so')) != []:
                atlas = True
                libdir = dir
                break
        satlas = False
        for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
            if glob(join(dir, 'libsatlas.so')) != []:
                satlas = True
                libdir = dir
                break
        openblas = False
        for dir in ['/usr/lib', '/usr/local/lib']:
            if glob(join(dir, 'libopenblas.so')) != []:
                openblas = True
                libdir = dir
                break
        if openblas:  # prefer openblas
            libraries += ['openblas']
            library_dirs += [libdir]
        else:
            if atlas:  # then atlas
                # http://math-atlas.sourceforge.net/errata.html#LINK
                # atlas does not respect OMP_NUM_THREADS - build single-thread
                # http://math-atlas.sourceforge.net/faq.html#tsafe
                libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                library_dirs += [libdir]
            elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                libraries += ['satlas']
                library_dirs += [libdir]
            else:
                libraries += ['blas', 'lapack']


def mtime(path, name, mtimes):
    """Return modification time.

    The modification time of a source file is returned.  If one of its
    dependencies is newer, the mtime of that file is returned.
    This function fails if two include files with the same name
    are present in different directories."""

    include = re.compile(r'^#\s*include "(\S+)"', re.MULTILINE)

    if name in mtimes:
        return mtimes[name]
    t = os.stat(os.path.join(path, name))[ST_MTIME]
    for name2 in include.findall(open(os.path.join(path, name)).read()):
        path2, name22 = os.path.split(name2)
        if name22 != name:
            t = max(t, mtime(os.path.join(path, path2), name22, mtimes))
    mtimes[name] = t
    return t


def check_dependencies(sources):
    # Distutils does not do deep dependencies correctly.  We take care of
    # that here so that "python setup.py build_ext" always does the right
    # thing!
    mtimes = {}  # modification times

    # Remove object files if any dependencies have changed:
    plat = distutils.util.get_platform() + '-' + sys.version[0:3]
    remove = False
    for source in sources:
        path, name = os.path.split(source)
        t = mtime(path + '/', name, mtimes)
        o = 'build/temp.%s/%s.o' % (plat, source[:-2])  # object file
        if os.path.exists(o) and t > os.stat(o)[ST_MTIME]:
            print('removing', o)
            os.remove(o)
            remove = True

    so = 'build/lib.%s/_gpaw.so' % plat
    if os.path.exists(so) and remove:
        # Remove shared object C-extension:
        # print 'removing', so
        os.remove(so)


def test_configuration():
    raise NotImplementedError


def write_configuration(define_macros, include_dirs, libraries, library_dirs,
                        extra_link_args, extra_compile_args,
                        runtime_library_dirs, extra_objects, mpicompiler,
                        mpi_libraries, mpi_library_dirs, mpi_include_dirs,
                        mpi_runtime_library_dirs, mpi_define_macros):

    # Write the compilation configuration into a file
    try:
        out = open('configuration.log', 'w')
    except IOError as x:
        print(x)
        return
    print("Current configuration", file=out)
    print("libraries", libraries, file=out)
    print("library_dirs", library_dirs, file=out)
    print("include_dirs", include_dirs, file=out)
    print("define_macros", define_macros, file=out)
    print("extra_link_args", extra_link_args, file=out)
    print("extra_compile_args", extra_compile_args, file=out)
    print("runtime_library_dirs", runtime_library_dirs, file=out)
    print("extra_objects", extra_objects, file=out)
    if mpicompiler is not None:
        print(file=out)
        print("Parallel configuration", file=out)
        print("mpicompiler", mpicompiler, file=out)
        print("mpi_libraries", mpi_libraries, file=out)
        print("mpi_library_dirs", mpi_library_dirs, file=out)
        print("mpi_include_dirs", mpi_include_dirs, file=out)
        print("mpi_define_macros", mpi_define_macros, file=out)
        print("mpi_runtime_library_dirs", mpi_runtime_library_dirs, file=out)
    out.close()


def build_interpreter(define_macros, include_dirs, libraries, library_dirs,
                      extra_link_args, extra_compile_args,
                      runtime_library_dirs, extra_objects,
                      mpicompiler, mpilinker, mpi_libraries, mpi_library_dirs,
                      mpi_include_dirs, mpi_runtime_library_dirs,
                      mpi_define_macros):

    # Build custom interpreter which is used for parallel calculations

    cfgDict = get_config_vars()
    plat = distutils.util.get_platform() + '-' + sys.version[0:3]

    cfiles = glob('c/[a-zA-Z_]*.c') + ['c/bmgs/bmgs.c']
    cfiles += glob('c/xc/*.c')
    # Make build process deterministic (for "reproducible build" in debian)
    # XXX some of this is duplicated in setup.py!  Why do the same thing twice?
    cfiles.sort()

    sources = ['c/bc.c', 'c/localized_functions.c', 'c/mpi.c', 'c/_gpaw.c',
               'c/operators.c', 'c/woperators.c', 'c/transformers.c',
               'c/elpa.c',
               'c/blacs.c', 'c/utilities.c', 'c/xc/libvdwxc.c']
    objects = ' '.join(['build/temp.%s/' % plat + x[:-1] + 'o'
                        for x in cfiles])

    if not os.path.isdir('build/bin.%s/' % plat):
        os.makedirs('build/bin.%s/' % plat)
    exefile = 'build/bin.%s/' % plat + '/gpaw-python'

    libraries += mpi_libraries
    library_dirs += mpi_library_dirs
    define_macros += mpi_define_macros
    include_dirs += mpi_include_dirs
    runtime_library_dirs += mpi_runtime_library_dirs

    define_macros.append(('PARALLEL', '1'))
    define_macros.append(('GPAW_INTERPRETER', '1'))
    macros = ' '.join(['-D%s=%s' % x for x in define_macros if x[0].strip()])

    include_dirs.append(cfgDict['INCLUDEPY'])
    include_dirs.append(cfgDict['CONFINCLUDEPY'])
    includes = ' '.join(['-I' + incdir for incdir in include_dirs])

    library_dirs.append(cfgDict['LIBPL'])
    lib_dirs = ' '.join(['-L' + lib for lib in library_dirs])

    libs = ' '.join(['-l' + lib for lib in libraries if lib.strip()])
    # LIBDIR/INSTSONAME will point at the static library if that is how
    # Python was compiled:
    lib = Path(cfgDict['LIBDIR']) / cfgDict['INSTSONAME']
    if lib.is_file():
        libs += ' {}'.format(lib)
    else:
        libs += ' ' + cfgDict.get('BLDLIBRARY',
                                  '-lpython%s' % cfgDict['VERSION'])
    libs = ' '.join([libs, cfgDict['LIBS'], cfgDict['LIBM']])

    # Hack taken from distutils to determine option for runtime_libary_dirs
    if sys.platform[:6] == 'darwin':
        # MacOSX's linker doesn't understand the -R flag at all
        runtime_lib_option = '-L'
    elif sys.platform[:5] == 'hp-ux':
        runtime_lib_option = '+s -L'
    elif os.popen('mpicc --showme 2> /dev/null', 'r').read()[:3] == 'gcc':
        runtime_lib_option = '-Wl,-R'
    elif os.popen('mpicc -show 2> /dev/null', 'r').read()[:3] == 'gcc':
        runtime_lib_option = '-Wl,-R'
    else:
        runtime_lib_option = '-R'

    runtime_libs = ' '.join([runtime_lib_option + lib
                             for lib in runtime_library_dirs])

    extra_link_args.append(cfgDict['LDFLAGS'])

    if sys.platform in ['aix5', 'aix6']:
        extra_link_args.append(cfgDict['LINKFORSHARED'].replace(
            'Modules', cfgDict['LIBPL']))
    elif sys.platform == 'darwin':
        # On a Mac, it is important to preserve the original compile args.
        # This should probably always be done ?!?
        extra_compile_args.append(cfgDict['CFLAGS'])
        extra_link_args.append(cfgDict['LINKFORSHARED'])
    else:
        extra_link_args.append(cfgDict['LINKFORSHARED'])

    extra_compile_args.append('-fPIC')

    # Compile the parallel sources
    for src in sources:
        obj = 'build/temp.%s/' % plat + src[:-1] + 'o'
        cmd = ('%s %s %s %s -o %s -c %s ') % \
              (mpicompiler,
               macros,
               ' '.join(extra_compile_args),
               includes,
               obj,
               src)
        print(cmd)
        error = os.system(cmd)
        if error != 0:
            return error

    # Link the custom interpreter
    cmd = ('%s -o %s %s %s %s %s %s %s') % \
          (mpilinker,
           exefile,
           objects,
           ' '.join(extra_objects),
           lib_dirs,
           libs,
           runtime_libs,
           ' '.join(extra_link_args))

    print(cmd)
    error = os.system(cmd)
    return error



# flake8: noqa
import sys
import time
import numpy as np

from gpaw.mpi import world
from gpaw.test.parunittest import ParallelTestCase, ParallelTextTestRunner, \
     defaultParallelTestLoader

# ------------------------------------------------------------------

class UTParallel(ParallelTestCase):
    """Parallel test suite of parallel test suite (don't ask)."""

    def reset_times(self, minwait=0.1, maxwait=0.2):
        if world.size == 1:
            return
        self.time_r = np.random.uniform(minwait, maxwait, size=world.size)
        world.broadcast(self.time_r, 0)

    def sleep_times(self):
        if world.size == 1:
            return
        time.sleep(self.time_r[world.rank])


class UTParallel_Succeeds(UTParallel):
    __doc__ = UTParallel.__doc__
    passes = True
    numtests = 1
    numerrors = 0
    numfails = 0

    def test_all_passes(self):
        self.reset_times()
        self.assertTrue(True)
        self.sleep_times()


class UTParallel_Raises(UTParallel):
    __doc__ = UTParallel.__doc__
    passes = False
    numtests = 2
    numerrors = 2
    numfails = 0

    def test_master_raises(self):
        self.reset_times()
        if world.rank == 0:
            raise Exception('I am special!')
        self.sleep_times()

    # Nichols A. Romero
    # naromero@alcf.anl.gov
    # Feb. 28, 2010
    # This test may not be valid
    # def test_slave_exits(self):
    #     self.reset_times()
    #     if world.rank == world.size-1:
    #         sys.exit(-1)
    #     self.sleep_times()

    def test_some_raises(self):
        self.reset_times()
        if world.rank % 3 == 0:
            raise RuntimeError
        elif world.rank % 3 == 1:
            raise IOError
        self.sleep_times()


class UTParallel_Fails(UTParallel):
    __doc__ = UTParallel.__doc__
    passes = False
    numtests = 4
    numerrors = 0
    numfails = 4

    def test_master_fails(self):
        self.reset_times()
        self.assertTrue(world.rank!=0)
        self.sleep_times()

    def test_all_fails(self):
        self.reset_times()
        self.assertTrue(False)
        self.sleep_times()

    def test_odd_fails(self):
        self.reset_times()
        self.assertTrue(world.size>1 and world.rank%2==0)
        self.sleep_times()

    def test_even_fails(self):
        self.reset_times()
        self.assertTrue(world.size>1 and world.rank%2==1)
        self.sleep_times()


# ------------------------------------------------------------------

if __name__ in ['__main__', '__builtin__']:
    # We may have been imported by test.py, if so we should redirect to logfile
    output = (__name__ == '__builtin__') and 'ut_parallel.log' or sys.stdout
    testrunner = ParallelTextTestRunner(stream=output, verbosity=2)

    testcases = [UTParallel_Succeeds, UTParallel_Raises, UTParallel_Fails]

    for test in testcases:
        info = '\n' + test.__name__ + '\n' + test.__doc__.strip('\n') + '\n'
        testsuite = defaultParallelTestLoader.loadTestsFromTestCase(test)
        testrunner.stream.writeln(info)
        testresult = testrunner.run(testsuite)
        verified = ((testresult.wasSuccessful() == test.passes) and
            (testresult.testsRun == test.numtests) and
            (len(testresult.errors) == test.numerrors) and
            (len(testresult.failures) == test.numfails))
        if verified:
            conclusion = 'STATUS: %s performed as expected.' % test
        else:
            conclusion = 'STATUS: %s did not meet expectations.\n\n' \
                ' ** Number of tests:    %2d ran,    %2d expected.\n%s\n' \
                ' ** Number of errors:   %2d raised, %2d expected.\n%s\n' \
                ' ** Number of failures: %2d failed, %2d expected.\n%s' \
                % (test, testresult.testsRun, test.numtests, testsuite, \
                   len(testresult.errors), test.numerrors, testresult.errors, \
                   len(testresult.failures), test.numfails, testresult.failures)

        separator0 = '*'*len(testresult.separator1)
        list(map(testrunner.stream.writeln, ['',separator0,conclusion,separator0,'']))

        # Provide feedback on failed tests if imported by test.py
        if __name__ == '__builtin__' and not verified:
            raise SystemExit('Test failed. Check ut_parallel.log for details.')




from __future__ import print_function

import os
import sys
import pickle
from math import pi

import numpy as np
from ase.units import Hartree, Bohr

import gpaw.mpi as mpi

from gpaw.response.chi0 import Chi0

from gpaw.response.kernels import get_coulomb_kernel
from gpaw.response.wstc import WignerSeitzTruncatedCoulomb
from gpaw.response.fxc import get_xc_kernel


class DielectricFunction:
    """This class defines dielectric function related physical quantities."""
    
    def __init__(self, calc, response='density',
                 name=None, frequencies=None, domega0=0.1,
                 omega2=10.0, omegamax=None, ecut=50,
                 gammacentered=False, hilbert=True,
                 nbands=None, eta=0.2, ftol=1e-6, threshold=1,
                 intraband=True, nblocks=1, world=mpi.world, txt=sys.stdout,
                 gate_voltage=None, truncation=None, disable_point_group=False,
                 disable_time_reversal=False,
                 integrationmode=None, pbc=None, rate=0.0,
                 omegacutlower=None, omegacutupper=None, eshift=0.0):
        """Creates a DielectricFunction object.

        calc: str
            The groundstate calculation file that the linear response
            calculation is based on.
        response : str
            Type of response function. Currently collinear, scalar options
            'density', '+-' and '-+' are implemented. (move to general rf.py)
        name: str
            If defined, save the response function to::

                name + '%+d%+d%+d.pckl' % tuple((q_c * kd.N_c).round())

            where q_c is the reduced momentum and N_c is the number of
            kpoints along each direction.
        frequencies: np.ndarray
            Specification of frequency grid. If not set the non-linear
            frequency grid is used.
        domega0: float
            Frequency grid spacing for non-linear frequency grid at omega = 0.
        omega2: float
            Frequency at which the non-linear frequency grid has doubled
            the spacing.
        omegamax: float
            The upper frequency bound for the non-linear frequency grid.
        ecut: float
            Plane-wave cut-off.
        gammacentered: bool
            Center the grid of plane waves around the gamma point or q-vector
        hilbert: bool
            Use hilbert transform.
        nbands: int
            Number of bands from calc.
        eta: float
            Broadening parameter.
        ftol: float
            Threshold for including close to equally occupied orbitals,
            f_ik - f_jk > ftol.
        threshold: float
            Threshold for matrix elements in optical response perturbation
            theory.
        intraband: bool
            Include intraband transitions.
        world: comm
            mpi communicator.
        nblocks: int
            Split matrices in nblocks blocks and distribute them G-vectors or
            frequencies over processes.
        txt: str
            Output file.
        gate_voltage: float
            Shift Fermi level of ground state calculation by the
            specified amount.
        truncation: str
            'wigner-seitz' for Wigner Seitz truncated Coulomb.
            '2D, 1D or 0d for standard analytical truncation schemes.
            Non-periodic directions are determined from k-point grid
        eshift: float
            Shift unoccupied bands
        """

        self.chi0 = Chi0(calc, response=response, frequencies=frequencies,
                         domega0=domega0, omega2=omega2, omegamax=omegamax,
                         ecut=ecut, nbands=nbands, eta=eta,
                         gammacentered=gammacentered, hilbert=hilbert,
                         ftol=ftol, threshold=threshold,
                         intraband=intraband, world=world, nblocks=nblocks,
                         txt=txt, gate_voltage=gate_voltage,
                         disable_point_group=disable_point_group,
                         disable_time_reversal=disable_time_reversal,
                         integrationmode=integrationmode,
                         pbc=pbc, rate=rate, eshift=eshift)

        self.name = name

        self.omega_w = self.chi0.omega_w
        if omegacutlower is not None:
            inds_w = np.logical_and(self.omega_w > omegacutlower / Hartree,
                                    self.omega_w < omegacutupper / Hartree)
            self.omega_w = self.omega_w[inds_w]

        nw = len(self.omega_w)

        world = self.chi0.world
        self.mynw = (nw + world.size - 1) // world.size
        self.w1 = min(self.mynw * world.rank, nw)
        self.w2 = min(self.w1 + self.mynw, nw)
        self.truncation = truncation
    
    def calculate_chi0(self, q_c, spin='all'):
        """Calculates the response function.

        Calculate the response function for a specific momentum.

        q_c: [float, float, float]
            The momentum wavevector.
        spin : str or int
            If 'all' then include all spins.
            If 0 or 1, only include this specific spin.
            (not used in transverse reponse functions)
        """
        
        if self.name:
            kd = self.chi0.calc.wfs.kd
            name = self.name + '%+d%+d%+d.pckl' % tuple((q_c * kd.N_c).round())
            if os.path.isfile(name):
                return self.read(name)

        pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.chi0.calculate(q_c, spin)
        chi0_wGG = self.chi0.distribute_frequencies(chi0_wGG)
        self.chi0.timer.write(self.chi0.fd)
        if self.name:
            self.write(name, pd, chi0_wGG, chi0_wxvG, chi0_wvv)

        return pd, chi0_wGG, chi0_wxvG, chi0_wvv

    def write(self, name, pd, chi0_wGG, chi0_wxvG, chi0_wvv):
        nw = len(self.omega_w)
        nG = pd.ngmax
        world = self.chi0.world

        if world.rank == 0:
            fd = open(name, 'wb')
            pickle.dump((self.omega_w, pd, None, chi0_wxvG, chi0_wvv),
                        fd, pickle.HIGHEST_PROTOCOL)
            for chi0_GG in chi0_wGG:
                pickle.dump(chi0_GG, fd, pickle.HIGHEST_PROTOCOL)

            tmp_wGG = np.empty((self.mynw, nG, nG), complex)
            w1 = self.mynw
            for rank in range(1, world.size):
                w2 = min(w1 + self.mynw, nw)
                world.receive(tmp_wGG[:w2 - w1], rank)
                for w in range(w2 - w1):
                    pickle.dump(tmp_wGG[w], fd, pickle.HIGHEST_PROTOCOL)
                w1 = w2
            fd.close()
        else:
            world.send(chi0_wGG, 0)

    def read(self, name):
        print('Reading from', name, file=self.chi0.fd)
        fd = open(name, 'rb')
        omega_w, pd, chi0_wGG, chi0_wxvG, chi0_wvv = pickle.load(fd)
        for omega in self.omega_w:
            assert np.any(np.abs(omega - omega_w) < 1e-8)

        wmin = np.argmin(np.abs(np.min(self.omega_w) - omega_w))
        world = self.chi0.world

        nw = len(omega_w)
        nG = pd.ngmax

        if chi0_wGG is not None:
            # Old file format:
            chi0_wGG = chi0_wGG[wmin + self.w1:self.w2].copy()
        else:
            if world.rank == 0:
                chi0_wGG = np.empty((self.mynw, nG, nG), complex)
                for _ in range(wmin):
                    pickle.load(fd)
                for chi0_GG in chi0_wGG:
                    chi0_GG[:] = pickle.load(fd)
                tmp_wGG = np.empty((self.mynw, nG, nG), complex)
                w1 = self.mynw
                for rank in range(1, world.size):
                    w2 = min(w1 + self.mynw, nw)
                    for w in range(w2 - w1):
                        tmp_wGG[w] = pickle.load(fd)
                    world.send(tmp_wGG[:w2 - w1], rank)
                    w1 = w2
            else:
                chi0_wGG = np.empty((self.w2 - self.w1, nG, nG), complex)
                world.receive(chi0_wGG, 0)

        if chi0_wvv is not None:
            chi0_wxvG = chi0_wxvG[wmin:wmin + nw]
            chi0_wvv = chi0_wvv[wmin:wmin + nw]

        return pd, chi0_wGG, chi0_wxvG, chi0_wvv

    def collect(self, a_w):
        world = self.chi0.world
        b_w = np.zeros(self.mynw, a_w.dtype)
        b_w[:self.w2 - self.w1] = a_w
        nw = len(self.omega_w)
        A_w = np.empty(world.size * self.mynw, a_w.dtype)
        world.all_gather(b_w, A_w)
        return A_w[:nw]

    def get_frequencies(self):
        """ Return frequencies that Chi is evaluated on"""
        return self.omega_w * Hartree

    def get_chi(self, xc='RPA', q_c=[0, 0, 0], spin='all',
                direction='x', return_VchiV=True, q_v=None,
                rshe=0.99,
                spinpol_cut=None, density_cut=None, fxc_scaling=None):
        """ Returns v^1/2 chi v^1/2 for the density response and chi for the
        spin response. The truncated Coulomb interaction is included as
        v^-1/2 v_t v^-1/2. This is in order to conform with
        the head and wings of chi0, which is treated specially for q=0.
        
        spin : str or int
            If 'all' then include all spins.
            If 0 or 1, only include this specific spin.
            (not used in transverse reponse functions)
        rshe : float or None
            Expand kernel in real spherical harmonics inside augmentation
            spheres. If None, the kernel will be calculated without
            augmentation. The value of rshe (0<rshe<1) sets a convergence
            criteria for the expansion in real spherical harmonics.
        spinpol_cut : float
            cutoff spin polarization below which f_xc is evaluated in
            unpolarized limit (make sure divergent terms cancel out correctly)
        density_cut : float
            cutoff density below which f_xc is set to zero
        fxc_scaling : list
            Possible scaling of kernel to hit Goldstone mode.
            If w=0 is included in the present calculation and
            fxc_scaling=[True, None], the fxc_scaling to match
            kappaM_w[0] = 0. will be calculated. If
            fxc_scaling = [True, float], Kxc will be scaled by float.
            Default is None, i.e. no scaling
        """

        # XXX generalize to kernel check
        response = self.chi0.response
        if response in ['+-', '-+']:
            assert xc in ('ALDA_x', 'ALDA_X', 'ALDA')
        
        pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c, spin)
        
        if response == 'density':
            N_c = self.chi0.calc.wfs.kd.N_c
            
            Kbare_G = get_coulomb_kernel(pd,
                                         N_c,
                                         truncation=None,
                                         q_v=q_v)
            vsqr_G = Kbare_G**0.5
            nG = len(vsqr_G)
            
            if self.truncation is not None:
                if self.truncation == 'wigner-seitz':
                    self.wstc = WignerSeitzTruncatedCoulomb(pd.gd.cell_cv, N_c)
                else:
                    self.wstc = None
                Ktrunc_G = get_coulomb_kernel(pd,
                                              N_c,
                                              truncation=self.truncation,
                                              wstc=self.wstc,
                                              q_v=q_v)
                K_GG = np.diag(Ktrunc_G / Kbare_G)
            else:
                K_GG = np.eye(nG, dtype=complex)
               
            if pd.kd.gamma:
                if isinstance(direction, str):
                    d_v = {'x': [1, 0, 0],
                           'y': [0, 1, 0],
                           'z': [0, 0, 1]}[direction]
                else:
                    d_v = direction
                d_v = np.asarray(d_v) / np.linalg.norm(d_v)
                W = slice(self.w1, self.w2)
                chi0_wGG[:, 0] = np.dot(d_v, chi0_wxvG[W, 0])
                chi0_wGG[:, :, 0] = np.dot(d_v, chi0_wxvG[W, 1])
                chi0_wGG[:, 0, 0] = np.dot(d_v, np.dot(chi0_wvv[W], d_v).T)
            
            if xc != 'RPA':
                Kxc_GG = get_xc_kernel(pd,
                                       self.chi0,
                                       functional=xc,
                                       chi0_wGG=chi0_wGG,
                                       density_cut=density_cut)
                K_GG += Kxc_GG / vsqr_G / vsqr_G[:, np.newaxis]
               
            # Invert Dyson eq.
            chi_wGG = []
            for chi0_GG in chi0_wGG:
                """v^1/2 chi0 V^1/2"""
                chi0_GG[:] = chi0_GG * vsqr_G * vsqr_G[:, np.newaxis]
                chi_GG = np.dot(np.linalg.inv(np.eye(nG) -
                                              np.dot(chi0_GG, K_GG)),
                                chi0_GG)
                if not return_VchiV:
                    chi0_GG /= vsqr_G * vsqr_G[:, np.newaxis]
                    chi_GG /= vsqr_G * vsqr_G[:, np.newaxis]
                chi_wGG.append(chi_GG)
            
            if len(chi_wGG):
                chi_wGG = np.array(chi_wGG)
            else:
                chi_wGG = np.zeros((0, nG, nG), complex)
            
        # Spin response
        else:
            Kxc_GG = get_xc_kernel(pd,
                                   self.chi0,
                                   functional=xc,
                                   kernel=response[::-1],
                                   rshe=rshe,
                                   chi0_wGG=chi0_wGG,
                                   fxc_scaling=fxc_scaling,
                                   density_cut=density_cut,
                                   spinpol_cut=spinpol_cut)
            
            # Invert Dyson equation
            chi_wGG = []
            for chi0_GG in chi0_wGG:
                chi_GG = np.dot(np.linalg.inv(np.eye(len(chi0_GG)) -
                                              np.dot(chi0_GG, Kxc_GG)),
                                chi0_GG)
                
                chi_wGG.append(chi_GG)

        return pd, chi0_wGG, np.array(chi_wGG)
    
    def get_dynamic_susceptibility(self, xc='ALDA', q_c=[0, 0, 0],
                                   q_v=None,
                                   rshe=0.99,
                                   spinpol_cut=None, density_cut=None,
                                   fxc_scaling=None,
                                   filename='chiM_w.csv'):
        """Calculate the dynamic susceptibility.
         
        Returns macroscopic(could be generalized?) dynamic susceptibility:
        chiM0_w, chiM_w = DielectricFunction.get_dynamic_susceptibility()
        """
        
        pd, chi0_wGG, chi_wGG = self.get_chi(xc=xc, q_c=q_c,
                                             rshe=rshe,
                                             spinpol_cut=spinpol_cut,
                                             density_cut=density_cut,
                                             fxc_scaling=fxc_scaling)
        
        rf0_w = np.zeros(len(chi_wGG), dtype=complex)
        rf_w = np.zeros(len(chi_wGG), dtype=complex)
        
        for w, (chi0_GG, chi_GG) in enumerate(zip(chi0_wGG, chi_wGG)):
            rf0_w[w] = chi0_GG[0, 0]
            rf_w[w] = chi_GG[0, 0]

        rf0_w = self.collect(rf0_w)
        rf_w = self.collect(rf_w)

        if filename is not None and mpi.rank == 0:
            with open(filename, 'w') as fd:
                for omega, rf0, rf in zip(self.omega_w * Hartree, rf0_w, rf_w):
                    print('%.6f, %.6f, %.6f, %.6f, %.6f' %
                          (omega, rf0.real, rf0.imag, rf.real, rf.imag),
                          file=fd)

        return rf0_w, rf_w
    
    def get_dielectric_matrix(self, xc='RPA', q_c=[0, 0, 0],
                              direction='x', symmetric=True,
                              calculate_chi=False, q_v=None,
                              add_intraband=False):
        """Returns the symmetrized dielectric matrix.

        ::

            \tilde\epsilon_GG' = v^{-1/2}_G \epsilon_GG' v^{1/2}_G',

        where::

            epsilon_GG' = 1 - v_G * P_GG' and P_GG'

        is the polarization.

        ::

            In RPA:   P = chi^0
            In TDDFT: P = (1 - chi^0 * f_xc)^{-1} chi^0

        in addition to RPA one can use the kernels, ALDA, rALDA, rAPBE,
        Bootstrap and LRalpha (long-range kerne), where alpha is a user
        specified parameter (for example xc='LR0.25')

        The head of the inverse symmetrized dielectric matrix is equal
        to the head of the inverse dielectric matrix (inverse dielectric
        function)"""
        
        pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c)

        if add_intraband:
            print('add_intraband=True is not supported at this time')
            raise NotImplementedError

        N_c = self.chi0.calc.wfs.kd.N_c
        if self.truncation == 'wigner-seitz':
            self.wstc = WignerSeitzTruncatedCoulomb(pd.gd.cell_cv, N_c)
        else:
            self.wstc = None
        K_G = get_coulomb_kernel(pd,
                                 N_c,
                                 truncation=self.truncation,
                                 wstc=self.wstc,
                                 q_v=q_v)**0.5
        nG = len(K_G)

        if pd.kd.gamma:
            if isinstance(direction, str):
                d_v = {'x': [1, 0, 0],
                       'y': [0, 1, 0],
                       'z': [0, 0, 1]}[direction]
            else:
                d_v = direction

            d_v = np.asarray(d_v) / np.linalg.norm(d_v)
            W = slice(self.w1, self.w2)
            chi0_wGG[:, 0] = np.dot(d_v, chi0_wxvG[W, 0])
            chi0_wGG[:, :, 0] = np.dot(d_v, chi0_wxvG[W, 1])
            chi0_wGG[:, 0, 0] = np.dot(d_v, np.dot(chi0_wvv[W], d_v).T)
            if q_v is not None:
                print('Restoring q dependence of head and wings of chi0')
                chi0_wGG[:, 1:, 0] *= np.dot(q_v, d_v)
                chi0_wGG[:, 0, 1:] *= np.dot(q_v, d_v)
                chi0_wGG[:, 0, 0] *= np.dot(q_v, d_v)**2

        if xc != 'RPA':
            Kxc_GG = get_xc_kernel(pd,
                                   self.chi0,
                                   functional=xc,
                                   chi0_wGG=chi0_wGG)

        if calculate_chi:
            chi_wGG = []

        for chi0_GG in chi0_wGG:
            if xc == 'RPA':
                P_GG = chi0_GG
            else:
                P_GG = np.dot(np.linalg.inv(np.eye(nG) -
                                            np.dot(chi0_GG, Kxc_GG)),
                              chi0_GG)
            if symmetric:
                e_GG = np.eye(nG) - P_GG * K_G * K_G[:, np.newaxis]
            else:
                K_GG = (K_G**2 * np.ones([nG, nG])).T
                e_GG = np.eye(nG) - P_GG * K_GG
            if calculate_chi:
                K_GG = np.diag(K_G**2)
                if xc != 'RPA':
                    K_GG += Kxc_GG
                chi_wGG.append(np.dot(np.linalg.inv(np.eye(nG) -
                                                    np.dot(chi0_GG, K_GG)),
                                      chi0_GG))
            chi0_GG[:] = e_GG

        # chi0_wGG is now the dielectric matrix
        if calculate_chi:
            if len(chi_wGG):
                chi_wGG = np.array(chi_wGG)
            else:
                chi_wGG = np.zeros((0, nG, nG), complex)

        if not calculate_chi:
            return chi0_wGG
        else:
            # chi_wGG is the full density response function..
            return pd, chi0_wGG, chi_wGG

    def get_dielectric_function(self, xc='RPA', q_c=[0, 0, 0], q_v=None,
                                direction='x', filename='df.csv'):
        """Calculate the dielectric function.

        Returns dielectric function without and with local field correction:
        df_NLFC_w, df_LFC_w = DielectricFunction.get_dielectric_function()
        """
        e_wGG = self.get_dielectric_matrix(xc, q_c, direction, q_v=q_v)
        df_NLFC_w = np.zeros(len(e_wGG), dtype=complex)
        df_LFC_w = np.zeros(len(e_wGG), dtype=complex)

        for w, e_GG in enumerate(e_wGG):
            df_NLFC_w[w] = e_GG[0, 0]
            df_LFC_w[w] = 1 / np.linalg.inv(e_GG)[0, 0]

        df_NLFC_w = self.collect(df_NLFC_w)
        df_LFC_w = self.collect(df_LFC_w)

        if filename is not None and mpi.rank == 0:
            with open(filename, 'w') as fd:
                for omega, nlfc, lfc in zip(self.omega_w * Hartree,
                                            df_NLFC_w,
                                            df_LFC_w):
                    print('%.6f, %.6f, %.6f, %.6f, %.6f' %
                          (omega, nlfc.real, nlfc.imag, lfc.real, lfc.imag),
                          file=fd)

        return df_NLFC_w, df_LFC_w

    def get_macroscopic_dielectric_constant(self, xc='RPA',
                                            direction='x', q_v=None):
        """Calculate macroscopic dielectric constant.

        Returns eM_NLFC and eM_LFC.

        Macroscopic dielectric constant is defined as the real part
        of dielectric function at w=0.

        Parameters:

        eM_LFC: float
            Dielectric constant without local field correction. (RPA, ALDA)
        eM2_NLFC: float
            Dielectric constant with local field correction.
        """

        fd = self.chi0.fd
        print('', file=fd)
        print('%s Macroscopic Dielectric Constant:' % xc, file=fd)

        df_NLFC_w, df_LFC_w = self.get_dielectric_function(
            xc=xc,
            filename=None,
            direction=direction,
            q_v=q_v)
        eps0 = np.real(df_NLFC_w[0])
        eps = np.real(df_LFC_w[0])
        print('  %s direction' % direction, file=fd)
        print('    Without local field: %f' % eps0, file=fd)
        print('    Include local field: %f' % eps, file=fd)

        return eps0, eps

    def get_eels_spectrum(self, xc='RPA', q_c=[0, 0, 0],
                          direction='x', filename='eels.csv'):
        """Calculate EELS spectrum. By default, generate a file 'eels.csv'.

        EELS spectrum is obtained from the imaginary part of the
        density response function as, EELS(\omega) = - 4 * \pi / q^2 Im \chi.
        Returns EELS spectrum without and with local field corrections:

        df_NLFC_w, df_LFC_w = DielectricFunction.get_eels_spectrum()
        """
        
        # Calculate V^1/2 \chi V^1/2
        pd, Vchi0_wGG, Vchi_wGG = self.get_chi(xc=xc, q_c=q_c,
                                               direction=direction)
        Nw = self.omega_w.shape[0]

        # Calculate eels = -Im 4 \pi / q^2  \chi
        eels_NLFC_w = -(1. / (1. - Vchi0_wGG[:, 0, 0])).imag
        eels_LFC_w = - (Vchi_wGG[:, 0, 0]).imag

        # Collect frequencies
        eels_NLFC_w = self.collect(eels_NLFC_w)
        eels_LFC_w = self.collect(eels_LFC_w)

        # Write to file
        if filename is not None and mpi.rank == 0:
            fd = open(filename, 'w')
            print('# energy, eels_NLFC_w, eels_LFC_w', file=fd)
            for iw in range(Nw):
                print('%.6f, %.6f, %.6f' %
                      (self.chi0.omega_w[iw] * Hartree,
                       eels_NLFC_w[iw], eels_LFC_w[iw]), file=fd)
            fd.close()

        return eels_NLFC_w, eels_LFC_w

    def get_polarizability(self, xc='RPA', direction='x', q_c=[0, 0, 0],
                           filename='polarizability.csv', pbc=None):
        """Calculate the polarizability alpha.
        In 3D the imaginary part of the polarizability is related to the
        dielectric function by Im(eps_M) = 4 pi * Im(alpha). In systems
        with reduced dimensionality the converged value of alpha is
        independent of the cell volume. This is not the case for eps_M,
        which is ill defined. A truncated Coulomb kernel will always give
        eps_M = 1.0, whereas the polarizability maintains its structure.

        By default, generate a file 'polarizability.csv'. The five colomns are:
        frequency (eV), Real(alpha0), Imag(alpha0), Real(alpha), Imag(alpha)
        alpha0 is the result without local field effects and the
        dimension of alpha is \AA to the power of non-periodic directions
        """

        cell_cv = self.chi0.calc.wfs.gd.cell_cv
        if not pbc:
            pbc_c = self.chi0.calc.atoms.pbc
        else:
            pbc_c = np.array(pbc)
        if pbc_c.all():
            V = 1.0
        else:
            V = np.abs(np.linalg.det(cell_cv[~pbc_c][:, ~pbc_c]))

        if not self.truncation:
            """Standard expression for the polarizability"""
            df0_w, df_w = self.get_dielectric_function(xc=xc,
                                                       q_c=q_c,
                                                       filename=None,
                                                       direction=direction)
            alpha_w = V * (df_w - 1.0) / (4 * pi)
            alpha0_w = V * (df0_w - 1.0) / (4 * pi)
        else:
            """Since eps_M = 1.0 for a truncated Coulomb interaction, it does
            not make sense to apply it here. Instead one should define the
            polarizability by

                alpha * eps_M^{-1} = -L / (4 * pi) * <v_ind>

            where <v_ind> = 4 * pi * \chi / q^2 is the averaged induced
            potential (relative to the strength of the  external potential).
            With the bare Coulomb potential, this expression is equivalent to
            the standard one. In a 2D system \chi should be calculated with a
            truncated Coulomb potential and eps_M = 1.0"""

            print('Using truncated Coulomb interaction', file=self.chi0.fd)

            pd, chi0_wGG, chi_wGG = self.get_chi(xc=xc,
                                                 q_c=q_c,
                                                 direction=direction)
            alpha_w = -V * (chi_wGG[:, 0, 0]) / (4 * pi)
            alpha0_w = -V * (chi0_wGG[:, 0, 0]) / (4 * pi)

            alpha_w = self.collect(alpha_w)
            alpha0_w = self.collect(alpha0_w)

        Nw = len(alpha_w)
        if filename is not None and mpi.rank == 0:
            fd = open(filename, 'w')
            for iw in range(Nw):
                print('%.6f, %.6f, %.6f, %.6f, %.6f' %
                      (self.chi0.omega_w[iw] * Hartree,
                       alpha0_w[iw].real * Bohr**(sum(~pbc_c)),
                       alpha0_w[iw].imag * Bohr**(sum(~pbc_c)),
                       alpha_w[iw].real * Bohr**(sum(~pbc_c)),
                       alpha_w[iw].imag * Bohr**(sum(~pbc_c))), file=fd)
            fd.close()

        return alpha0_w * Bohr**(sum(~pbc_c)), alpha_w * Bohr**(sum(~pbc_c))

    def check_sum_rule(self, spectrum=None):
        """Check f-sum rule.

        It takes the y of a spectrum as an entry and it check its integral.

        spectrum: np.ndarray
            Input spectrum

        Note: not tested for spin response
        """

        assert (self.omega_w[1:] - self.omega_w[:-1]).ptp() < 1e-10

        fd = self.chi0.fd

        if spectrum is None:
            raise ValueError('No spectrum input ')
        dw = self.chi0.omega_w[1] - self.chi0.omega_w[0]
        N1 = 0
        for iw in range(len(spectrum)):
            w = iw * dw
            N1 += spectrum[iw] * w
        N1 *= dw * self.chi0.vol / (2 * pi**2)

        print('', file=fd)
        print('Sum rule:', file=fd)
        nv = self.chi0.calc.wfs.nvalence
        print('N1 = %f, %f  %% error' % (N1, (N1 - nv) / nv * 100), file=fd)

    def get_eigenmodes(self, q_c=[0, 0, 0], w_max=None, name=None,
                       eigenvalue_only=False, direction='x',
                       checkphase=True):
        """Plasmon eigenmodes as eigenvectors of the dielectric matrix."""
        
        assert self.chi0.world.size == 1

        pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c)
        e_wGG = self.get_dielectric_matrix(xc='RPA', q_c=q_c,
                                           direction=direction,
                                           symmetric=False)

        kd = pd.kd

        # Get real space grid for plasmon modes:
        r = pd.gd.get_grid_point_coordinates()
        w_w = self.omega_w * Hartree
        if w_max:
            w_w = w_w[np.where(w_w < w_max)]
        Nw = len(w_w)
        nG = e_wGG.shape[1]

        eig = np.zeros([Nw, nG], dtype=complex)
        eig_all = np.zeros([Nw, nG], dtype=complex)

        # Find eigenvalues and eigenvectors:
        e_GG = e_wGG[0]
        eig_all[0], vec = np.linalg.eig(e_GG)
        eig[0] = eig_all[0]
        vec_dual = np.linalg.inv(vec)
        omega0 = np.array([])
        eigen0 = np.array([], dtype=complex)
        v_ind = np.zeros([0, r.shape[1], r.shape[2], r.shape[3]],
                         dtype=complex)
        n_ind = np.zeros([0, r.shape[1], r.shape[2], r.shape[3]],
                         dtype=complex)

        # Loop to find the eigenvalues that crosses zero
        # from negative to positive values:
        for i in np.array(range(1, Nw)):
            e_GG = e_wGG[i]  # epsilon_GG'(omega + d-omega)
            eig_all[i], vec_p = np.linalg.eig(e_GG)
            vec_dual_p = np.linalg.inv(vec_p)
            overlap = np.abs(np.dot(vec_dual, vec_p))
            index = list(np.argsort(overlap)[:, -1])
            if len(np.unique(index)) < nG:  # add missing indices
                addlist = []
                removelist = []
                for j in range(nG):
                    if index.count(j) < 1:
                        addlist.append(j)
                    if index.count(j) > 1:
                        for l in range(1, index.count(j)):
                            removelist += \
                                list(np.argwhere(np.array(index) == j)[l])
                for j in range(len(addlist)):
                    index[removelist[j]] = addlist[j]

            vec = vec_p[:, index]
            vec_dual = vec_dual_p[index, :]
            eig[i] = eig_all[i, index]
            for k in [k for k in range(nG)
                      # Eigenvalue crossing:
                      if (eig[i - 1, k] < 0 and eig[i, k] > 0)]:
                a = np.real((eig[i, k] - eig[i - 1, k]) /
                            (w_w[i] - w_w[i - 1]))
                # linear interp for crossing point
                w0 = np.real(-eig[i - 1, k]) / a + w_w[i - 1]
                eig0 = a * (w0 - w_w[i - 1]) + eig[i - 1, k]
                print('crossing found at w = %1.2f eV' % w0)
                omega0 = np.append(omega0, w0)
                eigen0 = np.append(eigen0, eig0)

                # Fourier Transform:
                qG = pd.get_reciprocal_vectors(add_q=True)
                coef_G = np.diagonal(np.inner(qG, qG)) / (4 * pi)
                qGr_R = np.inner(qG, r.T).T
                factor = np.exp(1j * qGr_R)
                v_temp = np.dot(factor, vec[:, k])
                n_temp = np.dot(factor, vec[:, k] * coef_G)
                if checkphase:  # rotate eigenvectors in complex plane
                    integral = np.zeros([81])
                    phases = np.linspace(0, 2, 81)
                    for ip in range(81):
                        v_int = v_temp * np.exp(1j * pi * phases[ip])
                        integral[ip] = abs(np.imag(v_int)).sum()
                    phase = phases[np.argsort(integral)][0]
                    v_temp *= np.exp(1j * pi * phase)
                    n_temp *= np.exp(1j * pi * phase)
                v_ind = np.append(v_ind, v_temp[np.newaxis, :], axis=0)
                n_ind = np.append(n_ind, n_temp[np.newaxis, :], axis=0)

        kd = self.chi0.calc.wfs.kd
        if name is None and self.name:
            name = (self.name + '%+d%+d%+d-eigenmodes.pckl' %
                    tuple((q_c * kd.N_c).round()))
        elif name:
            name = (name + '%+d%+d%+d-eigenmodes.pckl' %
                    tuple((q_c * kd.N_c).round()))
        else:
            name = '%+d%+d%+d-eigenmodes.pckl' % tuple((q_c * kd.N_c).round())

        # Returns: real space grid, frequency grid,
        # sorted eigenvalues, zero-crossing frequencies + eigenvalues,
        # induced potential + density in real space.
        if eigenvalue_only:
            pickle.dump((r * Bohr, w_w, eig),
                        open(name, 'wb'), pickle.HIGHEST_PROTOCOL)
            return r * Bohr, w_w, eig
        else:
            pickle.dump((r * Bohr, w_w, eig, omega0, eigen0,
                         v_ind, n_ind), open(name, 'wb'),
                        pickle.HIGHEST_PROTOCOL)
            return r * Bohr, w_w, eig, omega0, eigen0, v_ind, n_ind

    def get_spatial_eels(self, q_c=[0, 0, 0], direction='x',
                         w_max=None, filename='eels', r=None, perpdir=None):
        """Spatially resolved loss spectrum.

        The spatially resolved loss spectrum is calculated as the inverse
        fourier transform of ``VChiV = (eps^{-1}-I)V``::

            EELS(w,r) = - Im [sum_{G,G'} e^{iGr} Vchi_{GG'}(w) V_G'e^{-iG'r}]
                          \delta(w-G\dot v_e )

        Input parameters:

        direction: 'x', 'y', or 'z'
            The direction for scanning acroos the structure
            (perpendicular to the electron beam) .
        w_max: float
            maximum frequency
        filename: str
            name of output

        Returns: real space grid, frequency points, EELS(w,r)
        """
        
        assert self.chi0.world.size == 1

        pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c)
        e_wGG = self.get_dielectric_matrix(xc='RPA', q_c=q_c,
                                           symmetric=False)

        if r is None:
            r = pd.gd.get_grid_point_coordinates()
            ix = r.shape[1] // 2 * 0
            iy = r.shape[2] // 2 * 0
            iz = r.shape[3] // 2
            if direction == 'x':
                r = r[:, :, iy, iz]
                perpdir = [1, 2]
            if direction == 'y':
                r = r[:, ix, :, iz]
                perpdir = [0, 2]
            if direction == 'z':
                r = r[:, ix, iy, :]
                perpdir = [0, 1]

        nG = e_wGG.shape[1]
        Gvec = pd.G_Qv[pd.Q_qG[0]]
        Glist = []

        # Only use G-vectors that are zero along electron beam
        # due to \delta(w-G\dot v_e )
        q_v = pd.K_qv[0]
        for iG in range(nG):
            if perpdir is not None:
                if Gvec[iG, perpdir[0]] == 0 and Gvec[iG, perpdir[1]] == 0:
                    Glist.append(iG)
            elif not np.abs(np.dot(q_v, Gvec[iG])) < \
                    np.linalg.norm(q_v) * np.linalg.norm(Gvec[iG]):
                Glist.append(iG)
        qG = Gvec[Glist] + pd.K_qv

        w_w = self.omega_w * Hartree
        if w_max:
            w_w = w_w[np.where(w_w < w_max)]
        Nw = len(w_w)
        qGr = np.inner(qG, r.T).T
        phase = np.exp(1j * qGr)
        V_G = (4 * pi) / np.diagonal(np.inner(qG, qG))
        phase2 = np.exp(-1j * qGr) * V_G
        E_wrr = np.zeros([Nw, r.shape[1], r.shape[1]])
        E_wr = np.zeros([Nw, r.shape[1]])
        Eavg_w = np.zeros([Nw], complex)
        Ec_wr = np.zeros([Nw, r.shape[1]], complex)
        for i in range(Nw):
            Vchi_GG = (np.linalg.inv(e_wGG[i]) -
                       np.eye(nG))[Glist, :][:, Glist]

            qG_G = np.sum(qG**2, axis=1)**0.5
            Eavg_w[i] = np.trace(Vchi_GG * np.diag(V_G * qG_G))

            # Fourier transform:
            E_wrr[i] = -np.imag(np.dot(np.dot(phase, Vchi_GG), phase2.T))
            E_wr[i] = np.diagonal(E_wrr[i])
            Ec_wr[i] = np.diagonal(np.dot(np.dot(phase, Vchi_GG *
                                                 np.diag(qG_G)), phase2.T))
        pickle.dump((r * Bohr, w_w, E_wr), open('%s.pickle' % filename, 'wb'),
                    pickle.HIGHEST_PROTOCOL)

        return r * Bohr, w_w, E_wr, Ec_wr, Eavg_w



"""Test Hirshfeld for spin/no spin consitency"""
from ase import Atom
from gpaw import GPAW
from ase.parallel import parprint
from gpaw.analyse.hirshfeld import HirshfeldPartitioning
from gpaw import FermiDirac
from gpaw.cluster import Cluster
from gpaw.test import equal

h = 0.25
box = 3

atoms = Cluster()
atoms.append(Atom('H'))
atoms.minimal_box(box)

volumes = []
for spinpol in [False, True]:
    calc = GPAW(h=h,
                occupations=FermiDirac(0.1, fixmagmom=True),
                experimental={'niter_fixdensity': 2},
                spinpol=spinpol)
    calc.calculate(atoms)
    volumes.append(HirshfeldPartitioning(calc).get_effective_volume_ratios())
parprint(volumes)
equal(volumes[0][0], volumes[1][0], 4e-9)



from ase import Atom, Atoms

from gpaw import GPAW, restart, Davidson, Mixer
from gpaw.atom.generator import Generator
from gpaw.atom.configurations import parameters
from gpaw import setup_paths
from gpaw.mpi import world
from gpaw.test import equal

# This test calculates the derivative discontinuity of Ne-atom
# first on 3D without restart. Then does restart and recalculates.

atom = 'Ne'
setup_paths.insert(0, '.')

for xcname in ['GLLB','GLLBSC']:
    if world.rank == 0:
        g = Generator(atom, xcname =xcname, scalarrel=False,nofiles=True)
        g.run(**parameters[atom])
        eps = g.e_j[-1]
    world.barrier()

    a = 10
    Ne = Atoms([Atom(atom, (0, 0, 0))],
               cell=(a, a, a), pbc=False)
    Ne.center()
    calc = GPAW(eigensolver=Davidson(4), nbands=10, h=0.18, xc=xcname,
                basis='dzp',
                mixer=Mixer(0.6))
    Ne.set_calculator(calc)
    e = Ne.get_potential_energy()
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    response.calculate_delta_xc()
    KS, dxc = response.calculate_delta_xc_perturbation()
    if xcname=='GLLB':
        equal(KS+dxc, 24.89, 1.5e-1)
    else:
        equal(KS+dxc, 27.70, 6.0e-2)
    eps3d = calc.wfs.kpt_u[0].eps_n[3]
    calc.write('Ne_temp.gpw')

    atoms, calc = restart('Ne_temp.gpw')
    KS2, dxc2 = response.calculate_delta_xc_perturbation()
    equal(KS, KS2, 1e-5)
    equal(dxc2, dxc, 1e-5)

    # Hardness of Ne 24.71eV by GLLB+Dxc, experimental I-A = I = 21.56eV
    #
    # Not sure where 24.71 comes from, but with better grid and better
    # stencil, result becomes 24.89.  --askhl

    if world.rank == 0:
        equal(eps, eps3d, 1e-3)
    if xcname=='GLLB':
        equal(24.89, KS2+dxc2, 1.2e-1)



from __future__ import print_function

from ase import Atom, Atoms
from ase.parallel import size, rank

from gpaw import GPAW, FermiDirac
from gpaw.analyse.simple_stm import SimpleStm
from gpaw.test import equal

load = True
load = False
txt = '/dev/null'
txt = '-'

me = ''
if size > 1:
    me += 'rank ' + str(rank) + ': '

BH = Atoms([Atom('B', [.0, .0, .41]),
            Atom('H', [.0, .0, -1.23])], cell=[5, 6, 6.5])
BH.center()

f3dname = 'stm3d.plt'


def testSTM(calc):
    stm = SimpleStm(calc)
    stm.write_3D([1, 0, 0], f3dname)  # single wf
    wf = stm.gd.integrate(stm.ldos)
##    print "wf=", wf

    if size == 1:  # XXXX we have problem with reading plt in parallel
        stm2 = SimpleStm(f3dname)
        wf2 = stm2.gd.integrate(stm2.ldos)
        print('Integrals: written, read=', wf, wf2)
        equal(wf, wf2, 2.e-7)

    stm.scan_const_current(0.02, 5)
##    print eigenvalue_string(calc)
    stm.write_3D(3.1, f3dname)
    wf2 = stm.gd.integrate(stm.ldos)
##    print "wf2=", wf2
    equal(wf2, 2, 0.12)

    return wf

# finite system without spin and width
fname = 'BH-nospin_wfs.gpw'
if not load:
    BH.set_pbc(False)
    cf = GPAW(nbands=3, h=.3, txt=txt)
    BH.set_calculator(cf)
    e1 = BH.get_potential_energy()
    niter1 = cf.get_number_of_iterations()
    cf.write(fname, 'all')
else:
    cf = GPAW(fname, txt=txt)
wf = testSTM(cf)

# finite system with spin
fname = 'BH-spin_Sz2_wfs.gpw'
BH.set_initial_magnetic_moments([1, 1])
if not load:
    BH.set_pbc(False)
    cf = GPAW(occupations=FermiDirac(0.1, fixmagmom=True),
              nbands=5,
              h=0.3,
              txt=txt)
    BH.set_calculator(cf)
    e2 = BH.get_potential_energy()
    niter2 = cf.get_number_of_iterations()
    cf.write(fname, 'all')
else:
    cf = GPAW(fname, txt=txt)
testSTM(cf)

# periodic system
if not load:
    BH.set_pbc(True)
    cp = GPAW(spinpol=True, nbands=3, h=.3,
              kpts=(2, 1, 1), txt=txt, symmetry='off')
    BH.set_calculator(cp)
    e3 = BH.get_potential_energy()
    niter3 = cp.get_number_of_iterations()
    cp.write('BH-4kpts_wfs.gpw', 'all')
else:
    cp = GPAW('BH-4kpts_wfs.gpw', txt=txt)

stmp = SimpleStm(cp)

stmp.write_3D(-4., f3dname)
print(me + 'Integrals(occ): 2 * wf, bias=', 2 * wf,
      stmp.gd.integrate(stmp.ldos))
equal(2 * wf, stmp.gd.integrate(stmp.ldos), 0.02)

stmp.write_3D(+4., f3dname)
print(me + 'Integrals(unocc): 2 * wf, bias=', end=' ')
print(2 * wf, stmp.gd.integrate(stmp.ldos))
equal(2 * wf, stmp.gd.integrate(stmp.ldos), 0.02)

energy_tolerance = 0.007
niter_tolerance = 0
equal(e1, -2.54026, energy_tolerance)
equal(e2, -1.51101, energy_tolerance)
equal(e3, -2.83573, energy_tolerance)



import numpy as np

from ase import Atoms
from ase.parallel import parprint
from ase.units import Hartree

import gpaw.mpi as mpi
from gpaw import GPAW
from gpaw.test import equal
from gpaw.lrtddft.kssingle import KSSingles


Be = Atoms('Be')
Be.center(vacuum=4)
if 1:
    # introduce a sligth non-orthgonality
    cell = Be.get_cell()
    cell[1] += 0.001 * cell[0]
    Be.set_cell(cell)

txt = None
#txt='-'
eigensolver = None
#eigensolver = 'rmm-diis'

#modes = ['lcao', 'fd']
modes = ['fd']

for mode in modes:
    energy = {}
    osz = {}
    for pbc in [False, True]:
        Be.set_pbc(pbc)
        if pbc:
            name = 'periodic'
            calc = GPAW(h=0.25, nbands=4, kpts=(1,2,2), mode=mode,
                        poissonsolver={'name': 'fd'},
                        symmetry='off',
                        eigensolver=eigensolver, txt=txt)
        else:
            name = 'zero_bc'
            calc = GPAW(h=0.25, nbands=4, mode=mode,
                        poissonsolver={'name': 'fd'},
                        eigensolver=eigensolver, txt=txt)
        Be.set_calculator(calc)
        Be.get_potential_energy()

        kss = KSSingles(calc, eps=0.9)
        # all s->p transitions at the same energy [Ha] and
        # oscillator_strength
        for ks in kss:
            equal(ks.get_energy(), kss[0].get_energy(), 5.e-3)
            equal(ks.get_oscillator_strength()[0],
                  kss[0].get_oscillator_strength()[0], 5.e-3)
            equal(ks.get_oscillator_strength()[0],
                  ks.get_oscillator_strength()[1:].sum() / 3, 1.e-15)
            for c in range(3):
                equal(ks.get_oscillator_strength()[1 + c],
                      ks.get_dipole_tensor()[c, c], 1.e-15)
        energy[name] = np.array(
            [ks.get_energy() * Hartree for ks in kss]).mean()
        osz[name] = np.array(
            [ks.get_oscillator_strength()[0] for ks in kss]).sum()

        parprint(name + ':')
        parprint(kss)

        # I/O
        fname = 'kss_' + name + '.dat'
        kss.write(fname)
        mpi.world.barrier()
        kss = KSSingles(fname)
        kss1 = KSSingles(fname, jend=1)
        assert(len(kss1) == calc.wfs.kd.nks)

    # periodic and non-periodic should be roughly equal
    equal(energy['zero_bc'], energy['periodic'], 5.e-2)
    equal(osz['zero_bc'], osz['periodic'], 2.e-2)



# This test verifies that the FastPoissonSolver produces solutions to
# the Poisson equation with very small residuals for different cells,
# pbcs, and grids.

import itertools
import numpy as np
from ase.build import bulk
from gpaw.poisson import FastPoissonSolver, BadAxesError
from gpaw.grid_descriptor import GridDescriptor
from gpaw.fd_operators import Laplace
from gpaw.mpi import world
from gpaw.utilities import h2gpts

# Test: different pbcs
# For pbc=000, test charged system
# Different cells (orthorhombic/general)
# use_cholesky keyword


cell_cv = np.array(bulk('Au').cell)
rng = np.random.RandomState(42)

tf = range(2)
comm = world


def icells():
    # cells: orthorhombic fcc bcc hcp
    yield 'diag', np.diag([3., 4., 5.])

    from ase.build import fcc111
    atoms = fcc111('Au', size=(1, 1, 1))
    atoms.center(vacuum=1, axis=2)
    yield 'fcc111@z', atoms.cell.copy()
    yield 'fcc111@x', atoms.cell[[2, 0, 1]]
    yield 'fcc111@y', atoms.cell[[1, 2, 0]]

    for sym in ['Au', 'Fe', 'Sc']:
        cell = bulk(sym).cell
        yield sym, cell.copy()



#import matplotlib.pyplot as plt

tolerance = 1e-12

def test(cellno, cellname, cell_cv, idiv, pbc, nn):
    N_c = h2gpts(0.12, cell_cv, idiv=idiv)
    if idiv == 1:
        N_c += 1 - N_c % 2  # We want especially to test uneven grids
    gd = GridDescriptor(N_c, cell_cv, pbc_c=pbc)
    rho_g = gd.zeros()
    phi_g = gd.zeros()
    rho_g[:] = -0.3 + rng.rand(*rho_g.shape)

    # Neutralize charge:
    charge = gd.integrate(rho_g)
    magic = gd.get_size_of_global_array().prod()
    rho_g -= charge / gd.dv / magic
    charge = gd.integrate(rho_g)
    assert abs(charge) < 1e-12

    # Check use_cholesky=True/False ?
    from gpaw.poisson import FDPoissonSolver
    ps = FastPoissonSolver(nn=nn)
    #print('setgrid')

    # Will raise BadAxesError for some pbc/cell combinations
    ps.set_grid_descriptor(gd)

    ps.solve(phi_g, rho_g)

    laplace = Laplace(gd, scale=-1.0 / (4.0 * np.pi), n=nn)

    def get_residual_err(phi_g):
        rhotest_g = gd.zeros()
        laplace.apply(phi_g, rhotest_g)
        residual = np.abs(rhotest_g - rho_g)
        # Residual is not accurate at end of non-periodic directions
        # except for nn=1 (since effectively we use the right stencil
        # only for nn=1 at the boundary).
        #
        # To do this check correctly, the Laplacian should have lower
        # nn at the boundaries.  Therefore we do not test the residual
        # at these ends, only in between, by zeroing the bad ones:
        if nn > 1:
            exclude_points = nn - 1
            for c in range(3):
                if nn > 1 and not pbc[c]:
                    # get view ehere axis c refers becomes zeroth dimension:
                    X = residual.transpose(c, (c + 1) % 3, (c + 2) % 3)

                    if gd.beg_c[c] == 1:
                        X[:exclude_points] = 0.0
                    if gd.end_c[c] == gd.N_c[c]:
                        X[-exclude_points:] = 0.0
        return residual.max()

    maxerr = get_residual_err(phi_g)
    pbcstring = '{}{}{}'.format(*pbc)

    if 0:
        ps2 = FDPoissonSolver(relax='J', nn=nn, eps=1e-18)
        ps2.set_grid_descriptor(gd)
        phi2_g = gd.zeros()
        ps2.solve(phi2_g, rho_g)

        phimaxerr = np.abs(phi2_g - phi_g).max()
        maxerr2 = get_residual_err(phi2_g)
        msg = ('{:2d} {:8s} pbc={} err={:8.5e} err[J]={:8.5e} '
               'err[phi]={:8.5e} nn={:1d}'
               .format(cellno, cellname, pbcstring, maxerr, maxerr2,
                       phimaxerr, nn))

    state = 'ok' if maxerr < tolerance else 'FAIL'

    msg = ('{:2d} {:8s} grid={} pbc={} err[fast]={:8.5e} nn={:1d} {}'
           .format(cellno, cellname, N_c, pbcstring, maxerr, nn, state))
    if world.rank == 0:
        print(msg)

    return maxerr


errs = []
for idiv in [4, 1]:
    for cellno, (cellname, cell_cv) in enumerate(icells()):
        for pbc in itertools.product(tf, tf, tf):
            for nn in [1, 3]:
                try:
                    err = test(cellno, cellname, cell_cv, idiv, pbc, nn)
                except BadAxesError:
                    # Ignore incompatible pbc/cell combinations
                    continue

                errs.append(err)


for i, err in enumerate(errs):
    assert err < tolerance, err



from math import log
from ase import Atoms
from ase.io import read, write, iread
from ase.units import Bohr
from ase.db import connect
from gpaw import GPAW, FermiDirac
from gpaw.test import equal

a = 4.0
h = 0.2
hydrogen = Atoms('H',
                 [(a / 2, a / 2, a / 2)],
                 cell=(a, a, a))

hydrogen.calc = GPAW(h=h,
                     nbands=1,
                     convergence={'energy': 1e-7},
                     txt='h.txt')
e1 = hydrogen.get_potential_energy()
equal(e1, 0.526939, 0.001)

dens = hydrogen.calc.density
c = dens.gd.find_center(dens.nt_sG[0]) * Bohr
equal(abs(c - a / 2).max(), 0, 1e-13)

kT = 0.001
hydrogen.calc.set(occupations=FermiDirac(width=kT))
e2 = hydrogen.get_potential_energy()
equal(e1, e2 + log(2) * kT, 3.0e-7)

# Test ase.db a bit:
for name in ['h.json', 'h.db']:
    con = connect(name, append=False)
    con.write(hydrogen)
    id = con.write(hydrogen, foo='bar', data={'abc': [1, 2, 3]})
    assert id == 2
    assert con.reserve(foo='bar') is None
    row = con.get(foo='bar')
    assert row.energy == e2
    assert sum(row.data.abc) == 6
    del con[1]
    assert con.reserve(x=42) == 3

    write('x' + name, hydrogen)
    write('xx' + name, [hydrogen, hydrogen])

    assert read(name + '@foo=bar')[0].get_potential_energy() == e2
    for n, h in zip([1, 0], iread(name + '@:')):
        assert n == len(h)

# Test parsing of GPAW's text output:
h = read('h.txt')
error = abs(h.calc.get_eigenvalues() - hydrogen.calc.get_eigenvalues()).max()
assert error < 1e-5, error

# Test get_electrostatic_potential() method
v = hydrogen.calc.get_electrostatic_potential()
print(v.shape, v.ptp())
